﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Automation.Provider;
using System.Drawing;
using System.Windows;
using System.Windows.Automation;
using System.Windows.Forms;
using Syncfusion.Collections;
using System.ComponentModel;

namespace Syncfusion.Windows.Forms.Grid
{
    [Syncfusion.Documentation.DocumentationExclude()]
    public class GridUIAProvider : IRawElementProviderFragmentRoot, ISelectionProvider, IScrollProvider
    {
        GridControl Grid = null;
        GridDataBoundGrid DataGrid = null;


        public GridUIAProvider(Control grid)
        {
            if (grid is GridControl)
            {
                this.Grid = (GridControl)grid;
            }
            else if (grid is GridDataBoundGrid)
            {
                this.DataGrid = (GridDataBoundGrid)grid;
            }

        }

        #region IRawElementProviderFragmentRoot Members

        public IRawElementProviderFragment ElementProviderFromPoint(double x, double y)
        {
            System.Drawing.Point p = new System.Drawing.Point((int)x, (int)y);

            if (this.Grid != null)
            {
                p = Grid.PointToClient(p);

                GridRangeInfo info = this.Grid.PointToRangeInfo(p);

                return this.Grid[info.Top, info.Left].Provider;
            }
            else if (this.DataGrid != null)
            {
                p = DataGrid.PointToClient(p);

                GridRangeInfo info = this.DataGrid.PointToRangeInfo(p);

                return this.DataGrid[info.Top, info.Left].Provider;
            }
            else
                return null;
        }

        public IRawElementProviderFragment GetFocus()
        {
            if (this.Grid != null)
            {
                return this.Grid.Model[Grid.Model.CurrentCellInfo.RowIndex, Grid.Model.CurrentCellInfo.ColIndex].Provider;
            }
            else if (this.DataGrid != null)
            {
                return this.Grid.Model[Grid.Model.CurrentCellInfo.RowIndex, Grid.Model.CurrentCellInfo.ColIndex].Provider;
            }
            else
                return null;
        }

        #endregion

        #region IRawElementProviderFragment Members

        public System.Windows.Rect BoundingRectangle
        {
            get 
            {
                if (this.Grid != null)
                {
                    Rectangle rect = this.Grid.RectangleToScreen(this.Grid.ClientRectangle);
                    return new Rect(rect.X, rect.Y, rect.Width, rect.Height);
                }
                else if(this.DataGrid!=null)
                {
                    Rectangle rect = this.Grid.RectangleToScreen(this.Grid.ClientRectangle);
                    return new Rect(rect.X, rect.Y, rect.Width, rect.Height);
                }
                else
                {
                    return new Rect();
                }
            }
        }

        


        public IRawElementProviderFragmentRoot FragmentRoot
        {
            get 
            {
                return this;
            }
        }

        public IRawElementProviderSimple[] GetEmbeddedFragmentRoots()
        {
            return null;
        }

        public int[] GetRuntimeId()
        {
            if (this.Grid != null)
            {
                return new int[] { this.Grid.GetHashCode() };
            }
            else if (this.DataGrid != null)
            {
                return new int[] { this.DataGrid.GetHashCode() };
            }
            else
            {
                return null;
            }
        }

        public IRawElementProviderFragment Navigate(NavigateDirection direction)
        {
            switch (direction)
            {
                case NavigateDirection.FirstChild:
                    if (this.Grid != null)
                    {
                        return this.Grid.Model[0, 0].Provider;
                    }
                    else if (this.DataGrid != null)
                    {
                        return this.DataGrid.Model[0, 0].Provider;
                    }
                    else
                        return null;
                case NavigateDirection.LastChild:
                    return this.Grid.Model[this.Grid.Model.RowCount-1,this.Grid.Model.ColCount-1].Provider;
                default:
                    return null;
            }
        }

        public void SetFocus()
        {
            if (this.Grid != null)
                this.Grid.Focus();
            else if (this.DataGrid != null)
                this.DataGrid.Focus();
        }

        #endregion

        #region IRawElementProviderSimple Members

        
        public object GetPatternProvider(int patternId)
        {
            if (patternId == SelectionPatternIdentifiers.Pattern.Id)
            {
                return this;
            }
            else if (patternId == ValuePatternIdentifiers.Pattern.Id)
            {
                return this;
            }
            /* Not Implemented */
            //else if (patternId == ScrollPatternIdentifiers.Pattern.Id)
            //{
            //    return this;
            //}
            else
                return null;
        }

        public object GetPropertyValue(int propertyId)
        {
            if (propertyId == AutomationElementIdentifiers.ControlTypeProperty.Id)
            {
                return System.Windows.Automation.ControlType.Table.Id;
            }
            else
                return null;
        }

        public IRawElementProviderSimple HostRawElementProvider
        {
            get
            {
               return AutomationInteropProvider.HostProviderFromHandle(this.Grid.FindParentForm().Handle);
            }
        }

        public ProviderOptions ProviderOptions
        {
            get 
            {
                return System.Windows.Automation.Provider.ProviderOptions.ServerSideProvider| System.Windows.Automation.Provider.ProviderOptions.ServerSideProvider;
            }
        }

        #endregion

        #region ISelectionProvider Members

        public bool CanSelectMultiple
        {
            get { return true; }
        }

        /// <summary>
        /// Not Implemented.
        /// </summary>
        /// <returns></returns>
        public IRawElementProviderSimple[] GetSelection()
        {
            return null;
        }

        public bool IsSelectionRequired
        {
            get 
            { 
                return false; 
            }
        }

        #endregion

        #region IScrollProvider Members

        public double HorizontalScrollPercent
        {
            get { throw new NotImplementedException(); }
        }

        public double HorizontalViewSize
        {
            get { throw new NotImplementedException(); }
        }

        public bool HorizontallyScrollable
        {
            get { throw new NotImplementedException(); }
        }

        public void Scroll(System.Windows.Automation.ScrollAmount horizontalAmount, System.Windows.Automation.ScrollAmount verticalAmount)
        {
            throw new NotImplementedException();
        }

        public void SetScrollPercent(double horizontalPercent, double verticalPercent)
        {
            throw new NotImplementedException();
        }

        public double VerticalScrollPercent
        {
            get { throw new NotImplementedException(); }
        }

        public double VerticalViewSize
        {
            get { throw new NotImplementedException(); }
        }

        public bool VerticallyScrollable
        {
            get { throw new NotImplementedException(); }
        }

        #endregion




        
    }


    public class GridCellUIAProvider : IRawElementProviderFragment, IValueProvider, ISelectionItemProvider
    {
        GridControl Grid = null;
        GridDataBoundGrid DataGrid = null;
        GridStyleInfo Info = null;
        public GridCellUIAProvider(Control grid, GridStyleInfo info)
        {
            if (grid is GridControl)
            {
                this.Grid = (GridControl)grid;

            }
            else if (grid is GridDataBoundGrid)
            {
                this.DataGrid = (GridDataBoundGrid)grid;
            }
            this.Info = info;
        }

        
        #region IRawElementProviderFragment Members

        public System.Windows.Rect BoundingRectangle
        {
            get 
            {
                if (this.Grid != null)
                {
                    Rectangle r = Grid.RangeInfoToRectangle(GridRangeInfo.Cell(Info.CellIdentity.RowIndex, Info.CellIdentity.ColIndex));

                    r.Intersect(Grid.ClientRectangle);

                    r = Grid.RectangleToScreen(r);
                    return new Rect(r.X, r.Y, r.Width, r.Height);
                }
                else if (this.DataGrid != null)
                {
                    Rectangle r = DataGrid.RangeInfoToRectangle(GridRangeInfo.Cell(Info.CellIdentity.RowIndex, Info.CellIdentity.ColIndex));

                    r.Intersect(DataGrid.ClientRectangle);

                    r = DataGrid.RectangleToScreen(r);

                    return new Rect(r.X, r.Y, r.Width, r.Height);
                }
                else
                    return new Rect();
            }
        }

        
        public IRawElementProviderFragmentRoot FragmentRoot
        {
            get 
            {
                if (this.Grid != null)
                    return this.Grid.Provider;
                else if (this.DataGrid != null)
                    return this.DataGrid.Provider;
                else
                    return null;
            }
        }

        public IRawElementProviderSimple[] GetEmbeddedFragmentRoots()
        {
            return null;
        }

        public int[] GetRuntimeId()
        {
            return new int[] {Info.CellIdentity.RowIndex,Info.CellIdentity.ColIndex};
        }

        public IRawElementProviderFragment Navigate(NavigateDirection direction)
        {
            switch(direction)
            {
                //case NavigateDirection.FirstChild:
                //    return this.Grid[0, 0].Provider;
                //case NavigateDirection.LastChild:
                //    return this.Grid[Grid.RowCount - 1, Grid.ColCount - 1].Provider;
                case NavigateDirection.NextSibling:
                    {
                        if (this.Grid != null)
                        {
                            if (Info.CellIdentity.ColIndex + 1 <= this.Grid.Model.ColCount)
                            {
                                return this.Grid[Info.CellIdentity.RowIndex, Info.CellIdentity.ColIndex + 1].Provider;
                            }
                            else if (Info.CellIdentity.RowIndex + 1 <= this.Grid.Model.RowCount)
                            {
                                return this.Grid[Info.CellIdentity.RowIndex + 1, 0].Provider;
                            }
                            else
                                return null;
                        }
                        else if (this.DataGrid != null)
                        {
                            if (Info.CellIdentity.ColIndex + 1 <= this.DataGrid.Model.ColCount)
                            {
                                return this.DataGrid[Info.CellIdentity.RowIndex, Info.CellIdentity.ColIndex + 1].Provider;
                            }
                            else if (Info.CellIdentity.RowIndex + 1 <= this.DataGrid.Model.RowCount)
                            {
                                return this.DataGrid[Info.CellIdentity.RowIndex + 1, 0].Provider;
                            }
                            else
                                return null;
                        }
                        else
                            return null;
                    }
                case NavigateDirection.PreviousSibling:
                    {
                        if (Info.CellIdentity.ColIndex > 0)
                        {
                            if (this.Grid != null)
                                return this.Grid[Info.CellIdentity.RowIndex, Info.CellIdentity.ColIndex - 1].Provider;
                            else if (this.DataGrid != null)
                                return this.DataGrid[Info.CellIdentity.RowIndex, Info.CellIdentity.ColIndex - 1].Provider;
                            else
                                return null;
                        }
                        else if (Info.CellIdentity.RowIndex > 0)
                        {
                            if (this.Grid != null)
                                return this.Grid[Info.CellIdentity.RowIndex, this.Grid.Model.ColCount - 1].Provider;
                            else if (this.DataGrid != null)
                                return this.Grid[Info.CellIdentity.RowIndex, this.Grid.Model.ColCount - 1].Provider;
                            else
                                return null;
                        }
                        else
                            return null;
                    }
                default:
                    return null;
            }
        }

        public void SetFocus()
        {

        }

        #endregion

        #region IRawElementProviderSimple Members

        public object GetPatternProvider(int patternId)
        {
            if (patternId == ValuePatternIdentifiers.Pattern.Id)
            {
                return this;
            }
            else if(patternId == SelectionItemPatternIdentifiers.Pattern.Id)
            {
                return this;
            }
            else
                return null;
        }

        public object GetPropertyValue(int propertyId)
        {
           
            if (propertyId == AutomationElementIdentifiers.HelpTextProperty.Id)
            {
                if (this.Info.CellType == "PushButton")
                    return this.Info.Description;
                else
                    return this.Info.CellValue;
            }
            else if (propertyId == AutomationElementIdentifiers.NameProperty.Id)
            {
                return "GridCell_R" + this.Info.CellIdentity.RowIndex + "_C" + this.Info.CellIdentity.ColIndex;
            }
            else if (propertyId == AutomationElementIdentifiers.LabeledByProperty.Id)
            {
                return null;
            }
            else if (propertyId == AutomationElementIdentifiers.IsExpandCollapsePatternAvailableProperty.Id)
            {
                return false;
            }
            else if (propertyId == AutomationElementIdentifiers.ControlTypeProperty.Id)
            {

                //GetStyleInfoProperty(this.Info, AutomationElementIdentifiers.ControlTypeProperty.ProgrammaticName, this.Grid.Handle);

                switch (Info.CellType)
                {
                    case "TextBox":
                        return System.Windows.Automation.ControlType.Text.Id;
                    case "CheckBox":
                        return System.Windows.Automation.ControlType.CheckBox.Id;
                    case "PushButton":
                        return System.Windows.Automation.ControlType.Button.Id;
                    case "NumericUpDown":
                        return System.Windows.Automation.ControlType.Spinner.Id;
                    case "ComboBox":
                        return System.Windows.Automation.ControlType.ComboBox.Id;
                    case "Header":
                    case "RowHeaderCell":
                    case "ColumnHeaderCell":
                        return System.Windows.Automation.ControlType.Header.Id;
                    case "ColorEdit":
                    case "MonthCalendar":
                    case "DropDownGrid":
                    default:
                        return System.Windows.Automation.ControlType.Custom.Id;
                }
            }
            else if (propertyId == AutomationElementIdentifiers.AutomationIdProperty.Id)
            {
                return this.Info.CellIdentity;
            }
            else if (propertyId == AutomationElementIdentifiers.HasKeyboardFocusProperty.Id)
            {
                if (this.Grid != null)
                {
                    return Grid.Focused && Grid.CurrentCell.RowIndex == Info.CellIdentity.RowIndex
                        && Grid.CurrentCell.ColIndex == Info.CellIdentity.ColIndex;
                }
                else if (this.DataGrid != null)
                {
                    return DataGrid.Focused && DataGrid.CurrentCell.RowIndex == Info.CellIdentity.RowIndex
                       && DataGrid.CurrentCell.ColIndex == Info.CellIdentity.ColIndex;
                }
                else
                    return null;
            }
            else if (propertyId == AutomationElementIdentifiers.ClickablePointProperty.Id)
            {
                return null;
            }
            else if (propertyId == AutomationElementIdentifiers.ItemStatusProperty.Id)
            {
                return "Un Identified";
            }
            else if (propertyId == AutomationElementIdentifiers.IsEnabledProperty.Id)
            {
                return true;
            }
            else if (propertyId == AutomationElementIdentifiers.IsKeyboardFocusableProperty.Id)
            {
                return true;
            }
            else if (propertyId == AutomationElementIdentifiers.FrameworkIdProperty.Id)
            {
                return "Syncfusion";
            }
            else if (propertyId == AutomationElementIdentifiers.IsControlElementProperty.Id)
            {
                return true;
            }
            else if (propertyId == AutomationElementIdentifiers.IsContentElementProperty.Id)
            {
                return true;
            }
            else if (propertyId == AutomationElementIdentifiers.IsOffscreenProperty.Id)
            {
                return false;
            }
            else if (propertyId == AutomationElementIdentifiers.ItemTypeProperty.Id)
            {
                return "Syncfusion GridCell";
            }
            else if (propertyId == AutomationElementIdentifiers.LocalizedControlTypeProperty.Id)
            {
                return "Syncfsuion GridCell";
            }

            return null;
        }
        public const int ProviderOptionUseComThreading = 0x20;
        public IRawElementProviderSimple HostRawElementProvider
        {
            get 
            {
                if (this.Grid != null)
                    return this.Grid.Provider;
                else if (this.DataGrid != null)
                    return this.DataGrid.Provider;
                else
                    return null;
            }
        }

        public ProviderOptions ProviderOptions
        {
            get { return (ProviderOptions)((int)ProviderOptions.ServerSideProvider | ProviderOptionUseComThreading); }//return System.Windows.Automation.Provider.ProviderOptions.ServerSideProvider; }
        }

        #endregion

        #region IValueProvider Members

        public bool IsReadOnly
        {
            get { return false; }
        }

        public void SetValue(string value)
        {
            Info.FormattedText = value;
        }

        public string Value
        {
            get { return Info.FormattedText; }
        }

        #endregion

        #region ISelectionItemProvider Members

        public void AddToSelection()
        {
            throw new NotImplementedException();
        }

        public bool IsSelected
        {
            get { return false; }
        }

        /* Not Implemented */
        public void RemoveFromSelection()
        {
            
        }

        /* Not Implemented */
        public void Select()
        {
            
        }

        public IRawElementProviderSimple SelectionContainer
        {
            get
            {
                if (this.Grid != null)
                    return Grid.Provider;
                else if (this.DataGrid != null)
                    return DataGrid.Provider;
                else
                    return null;
            }
        }

        #endregion

    }
}
