using System;
using System.ComponentModel;
using System.Resources;
using System.Globalization;
using System.Diagnostics;
using System.Collections;
using System.Threading;

// Tip:
// To simplify rebuilding this class while working on your project
// you should add the following External Tool in Developer Studio:
// 
//     Title: Resgen SR.TXT
//     Command: C:\WINNT\system32\CMD.EXE
//     Arguments: /c resgen SR.txt & srgen @srgen.ini & echo Done.
//     Initial Directory: $(ProjectDir)
//     Please enable "X Use Output for Window."
// 
// Additionally you should add a file srgen.ini to project that 
// contains command line arguments for your project.
//
// Example: 
// SRGen.ini
// sr.txt namespace:Samples.SRGenSample classname:SR

namespace Syncfusion.Windows.Forms.Localization
{
    /// <summary>
    ///    SR provides localized access to string resources specific 
    ///    from the assembly manifest Syncfusion.Windows.Forms.Localization.SR.resources
    /// </summary>
	sealed class SR 
	{
		// Fields
		private ResourceManager resources;
		private static SR loader = null;

		// Strings 
		internal const string Bottom = "Bottom";
		internal const string CategoryAction = "CategoryAction";
		internal const string CategoryAppearance = "CategoryAppearance";
		internal const string CategoryBehavior = "CategoryBehavior";
		internal const string CategoryData = "CategoryData";
		internal const string ColorEditorPaletteTab = "ColorEditorPaletteTab";
		internal const string ColorEditorStandardTab = "ColorEditorStandardTab";
		internal const string ColorEditorSystemTab = "ColorEditorSystemTab";
		internal const string Copy = "Copy";
		internal const string Cut = "Cut";
		internal const string Delete = "Delete";
		internal const string DescriptionMenuItemText = "DescriptionMenuItemText";
		internal const string Error = "Error";
		internal const string FewerButtonsItemsText = "FewerButtonsItemsText";
		internal const string GroupBarAddRemoveButton = "GroupBarAddRemoveButton";
		internal const string GroupBarDropDownToolTip = "GroupBarDropDownToolTip";
		internal const string GroupBarExpandButtonToolTip = "GroupBarExpandButtonToolTip";
		internal const string GroupBarMinimizeButtonToolTip = "GroupBarMinimizeButtonToolTip";
		internal const string GroupBarNavigationPaneTooltip = "GroupBarNavigationPaneTooltip";
		internal const string InvalidArgument = "InvalidArgument";
		internal const string InvalidLowBoundArgumentEx = "InvalidLowBoundArgumentEx";
		internal const string InvalidLowBoundArgumentEx2 = "InvalidLowBoundArgumentEx2";
		internal const string LeftEdge = "LeftEdge";
		internal const string ModName = "ModName";
		internal const string MoreButtonsItemsText = "MoreButtonsItemsText";
		internal const string PageDown = "PageDown";
		internal const string PageLeft = "PageLeft";
		internal const string PageRight = "PageRight";
		internal const string PageUp = "PageUp";
		internal const string Paste = "Paste";
		internal const string RecordNavigationBarAddNewRecord = "RecordNavigationBarAddNewRecord";
		internal const string RecordNavigationBarCurrentRecord = "RecordNavigationBarCurrentRecord";
		internal const string RecordNavigationBarFirstRecord = "RecordNavigationBarFirstRecord";
		internal const string RecordNavigationBarLastRecord = "RecordNavigationBarLastRecord";
		internal const string RecordNavigationBarNextRecord = "RecordNavigationBarNextRecord";
		internal const string RecordNavigationBarPreviousRecord = "RecordNavigationBarPreviousRecord";
		internal const string RecordNavigationBarRecord = "RecordNavigationBarRecord";
		internal const string RecordNavigationBarRecordCount = "RecordNavigationBarRecordCount";
		internal const string ResetMenuItemText = "ResetMenuItemText";
		internal const string RightEdge = "RightEdge";
		internal const string ScrollDown = "ScrollDown";
		internal const string ScrollHere = "ScrollHere";
		internal const string ScrollLeft = "ScrollLeft";
		internal const string ScrollRight = "ScrollRight";
		internal const string ScrollUp = "ScrollUp";
		internal const string SelectAll = "SelectAll";
		internal const string StyleCategoryAppearance = "StyleCategoryAppearance";
		internal const string TabBarControlAdd = "TabBarControlAdd";
		internal const string TabBarControlRemove = "TabBarControlRemove";
		internal const string TabBarPageOnTabBarPage = "TabBarPageOnTabBarPage";
		internal const string TabBarSplitterControlAdd = "TabBarSplitterControlAdd";
		internal const string TabBarSplitterControlRemove = "TabBarSplitterControlRemove";
		internal const string TabBarToolTipEmptyIcon = "TabBarToolTipEmptyIcon";
		internal const string Test = "Test";
		internal const string TextParseFailedFormat = "TextParseFailedFormat";
		internal const string Top = "Top";
		internal const string TopLeftBottomRight = "TopLeftBottomRight";
		internal const string Undo = "Undo";
        internal const string OK = "OK";
        internal const string Cancel = "Cancel";
        internal const string Abort = "Abort";
        internal const string Retry ="Retry";
        internal const string Ignore= "Ignore";
        internal const string Yes = "Yes";
        internal const string No = "No";
        internal const string Close = "Close";
        internal const string Help = "Help";
        internal const string TryAgain = "TryAgain";
        internal const string Continue = "Continue";


		private SR()  
		{
			this.resources = new ResourceManager(this.GetType());
		}

		// Methods
		private static SR GetLoader()  
		{
			lock(typeof(SR))
			{
				if (SR.loader == null)
					SR.loader = new SR();
				return SR.loader;
			}
		}

		// Methods

		public static string GetString(CultureInfo culture, string name, object args)
		{
			SR sr = SR.GetLoader();
			
			if (sr == null) 
				return null;
			try
			{
				if (LocalizationProvider.Provider != null)
				{
					String result = string.Empty;

					result = LocalizationProvider.Provider.GetLocalizedString(culture, name, args);

					if (result != string.Empty)
						return result;
				}
				return sr.resources.GetString(name, culture);
			}
			catch
			{
				return name;
			}
		}


		public static string GetString(string name)  
		{
			return SR.GetString(null, name);
		}


		public static string GetString(string name, params object[] args)  
		{
			return SR.GetString(null, name,args);
		}

		public static string GetString(string name, object ctrl)
		{
			return SR.GetString(null, name, ctrl);
		}

		public static string GetString(CultureInfo culture, string name)  
		{
			SR sr = SR.GetLoader();
			if (sr == null) 
				return null;
            try
            {
                if (LocalizationProvider.Provider != null)
                {
                    String result = string.Empty;

                    result = LocalizationProvider.Provider.GetLocalizedString(culture, name, null );

                    if (result != string.Empty)
                        return result;
                }
                return sr.resources.GetString(name, culture);
            }
            catch
            {
                return name;
            }
		}


		public static object GetObject(CultureInfo culture, string name)  
		{
			SR sr = SR.GetLoader();
			if (sr == null) 
				return null;
			return sr.resources.GetObject(name, culture);
		}


		public static object GetObject(string name)  
		{
			return SR.GetObject(null, name);
		}



		public static bool GetBoolean(CultureInfo culture, string name)  
		{
			bool value;
			SR sr = SR.GetLoader();
			object obj;
			value = false;
			if (sr != null) 
			{
				obj = sr.resources.GetObject(name, culture);
				if (obj is System.Boolean) 
					value = ((bool) obj);
			}
			return value;
		}


		public static bool GetBoolean(string name)  
		{
			return SR.GetBoolean(name);
		}


		public static byte GetByte(CultureInfo culture, string name)  
		{
			byte value;
			SR sr = SR.GetLoader();
			object obj;
			value = (byte)0;
			if (sr != null) 
			{
				obj = sr.resources.GetObject(name, culture);
				if (obj is System.Byte) 
					value = ((byte) obj);
			}
			return value;
		}


		public static byte GetByte(string name)  
		{
			return SR.GetByte(null, name);
		}


		public static char GetChar(CultureInfo culture, string name)  
		{
			char value;
			SR sr = SR.GetLoader();
			object obj;
			value = (char)0;
			if (sr != null) 
			{
				obj = sr.resources.GetObject(name, culture);
				if (obj is System.Char)
					value = (char) obj;
			}
			return value;
		}


		public static char GetChar(string name)  
		{
			return SR.GetChar(null, name);
		}


		public static double GetDouble(CultureInfo culture, string name)  
		{
			double value;
			SR sr = SR.GetLoader();
			object obj;
			value = 0.0;
			if (sr != null) 
			{
				obj = sr.resources.GetObject(name, culture);
				if (obj is System.Double) 
					value = ((double) obj);
			}
			return value;
		}


		public static double GetDouble(string name)  
		{
			return SR.GetDouble(null, name);
		}


		public static float GetFloat(CultureInfo culture, string name)  
		{
			float value;
			SR sr = SR.GetLoader();
			object obj;
			value = 0.0f;
			if (sr != null) 
			{
				obj = sr.resources.GetObject(name, culture);
				if (obj is System.Single) 
					value = ((float)obj);
			}
			return value;
		}


		public static float GetFloat(string name)  
		{
			return SR.GetFloat(null, name);
		}


		public static int GetInt(string name)  
		{
			return SR.GetInt(null, name);
		}


		public static int GetInt(CultureInfo culture, string name)  
		{
			int value;
			SR sr = SR.GetLoader();
			object obj;
			value = 0;
			if (sr != null) 
			{
				obj = sr.resources.GetObject(name, culture);
				if (obj is System.Int32) 
					value = ((int) obj);
			}
			return value;
		}


		public static long GetLong(string name)  
		{
			return SR.GetLong(null, name);
		}


		public static long GetLong(CultureInfo culture, string name)  
		{
			Int64 value;
			SR sr = SR.GetLoader();
			object obj;
			value = ((Int64) 0);
			if (sr != null) 
			{
				obj = sr.resources.GetObject(name, culture);
				if (obj is System.Int64)
					value = ((Int64) obj);
			}
			return value;
		}

		public static short GetShort(CultureInfo culture, string name)  
		{
			short value;
			SR sr = SR.GetLoader();
			object obj;
			value = (short)0;
			if (sr != null)
			{
				obj = sr.resources.GetObject(name, culture);
				if (obj is System.Int16)
					value = ((short) obj);
			}
			return value;
		}


		public static short GetShort(string name)  
		{
			return SR.GetShort(null, name);
		}
	}

	/// <summary>
	/// Specifies the category in which the property or event will be displayed in a visual designer.
	/// </summary>
	/// <remarks>
	/// This is a localized version of CategoryAttribute. The localized string will be loaded from the 
	/// assembly manifest Syncfusion.Windows.Forms.Localization.SR.resources
	/// </remarks>
	[AttributeUsage(AttributeTargets.All, AllowMultiple = false, Inherited = true)] 
	sealed class SRCategoryAttribute : CategoryAttribute
	{
		public SRCategoryAttribute(string category)
			: base(category)
		{
		} 
        
		protected override string GetLocalizedString(string value)
		{
			return SR.GetString(value);
		} 
	} 

	/// <summary>
	/// Specifies a description for a property or event.
	/// </summary>
	/// <remarks>
	/// This is a localized version of DescriptionAttribute. The localized string will be loaded from the 
	/// assembly manifest Syncfusion.Windows.Forms.Localization.SR.resources
	/// </remarks>
	[AttributeUsage(AttributeTargets.All, AllowMultiple = false, Inherited = true)] 
	sealed class SRDescriptionAttribute : DescriptionAttribute
	{
		private bool replaced = false;
        
		public SRDescriptionAttribute(string description)
			: base(description)
		{
		} 
        
		public override string Description 
		{ 
			get
			{
				if (!this.replaced) 
				{
					this.replaced = true;
					this.DescriptionValue = SR.GetString(base.Description);
				}
				return base.Description;
			} 
		}
	}	
} // end of namespace Syncfusion.Windows.Forms.Localization

namespace Syncfusion.Windows.Forms
{
    /// <summary>
    /// Defines a <see cref="ILocalizationProvider"/> interface.
    /// </summary>
    public interface ILocalizationProvider
    {
        string GetLocalizedString(CultureInfo culture, string name,object  ctrl);
    }

    /// <summary>
    /// Provides static methods and properties to localize the UI.
    /// </summary>
    public sealed class LocalizationProvider
    {
        #region Fields
        private static ILocalizationProvider provider;
        #endregion

        #region LocalizationProvider
        public static ILocalizationProvider Provider
        {
            get
            {
                return provider;
            }
            set
            {
                provider = value;
            }
        }
        #endregion
    }

    /// <summary>
    /// ResourceIdentifiers contains resource Ids specific to the Syncfusion.Shared.Base assembly. 
    /// </summary>
    public sealed class ResourceIdentifiers
    {
        #region String Constants
        public const string Bottom = "Bottom";
        public const string CategoryAction = "CategoryAction";
        public const string CategoryAppearance = "CategoryAppearance";
        public const string CategoryBehavior = "CategoryBehavior";
        public const string CategoryData = "CategoryData";
        public const string ColorEditorPaletteTab = "ColorEditorPaletteTab";
        public const string ColorEditorStandardTab = "ColorEditorStandardTab";
        public const string ColorEditorSystemTab = "ColorEditorSystemTab";
        public const string Copy = "Copy";
        public const string Cut = "Cut";
        public const string Delete = "Delete";
        public const string DescriptionMenuItemText = "DescriptionMenuItemText";
        public const string Error = "Error";
        public const string FewerButtonsItemsText = "FewerButtonsItemsText";
        public const string GroupBarAddRemoveButton = "GroupBarAddRemoveButton";
        public const string GroupBarDropDownToolTip = "GroupBarDropDownToolTip";
        public const string GroupBarExpandButtonToolTip = "GroupBarExpandButtonToolTip";
        public const string GroupBarMinimizeButtonToolTip = "GroupBarMinimizeButtonToolTip";
        public const string GroupBarNavigationPaneTooltip = "GroupBarNavigationPaneTooltip";
        public const string InvalidArgument = "InvalidArgument";
        public const string InvalidLowBoundArgumentEx = "InvalidLowBoundArgumentEx";
        public const string InvalidLowBoundArgumentEx2 = "InvalidLowBoundArgumentEx2";
        public const string LeftEdge = "LeftEdge";
        public const string ModName = "ModName";
        public const string MoreButtonsItemsText = "MoreButtonsItemsText";
        public const string PageDown = "PageDown";
        public const string PageLeft = "PageLeft";
        public const string PageRight = "PageRight";
        public const string PageUp = "PageUp";
        public const string Paste = "Paste";
        public const string RecordNavigationBarAddNewRecord = "RecordNavigationBarAddNewRecord";
        public const string RecordNavigationBarCurrentRecord = "RecordNavigationBarCurrentRecord";
        public const string RecordNavigationBarFirstRecord = "RecordNavigationBarFirstRecord";
        public const string RecordNavigationBarLastRecord = "RecordNavigationBarLastRecord";
        public const string RecordNavigationBarNextRecord = "RecordNavigationBarNextRecord";
        public const string RecordNavigationBarPreviousRecord = "RecordNavigationBarPreviousRecord";
        public const string RecordNavigationBarRecord = "RecordNavigationBarRecord";
        public const string RecordNavigationBarRecordCount = "RecordNavigationBarRecordCount";
        public const string ResetMenuItemText = "ResetMenuItemText";
        public const string RightEdge = "RightEdge";
        public const string ScrollDown = "ScrollDown";
        public const string ScrollHere = "ScrollHere";
        public const string ScrollLeft = "ScrollLeft";
        public const string ScrollRight = "ScrollRight";
        public const string ScrollUp = "ScrollUp";
        public const string SelectAll = "SelectAll";
        public const string StyleCategoryAppearance = "StyleCategoryAppearance";
        public const string TabBarControlAdd = "TabBarControlAdd";
        public const string TabBarControlRemove = "TabBarControlRemove";
        public const string TabBarPageOnTabBarPage = "TabBarPageOnTabBarPage";
        public const string TabBarSplitterControlAdd = "TabBarSplitterControlAdd";
        public const string TabBarSplitterControlRemove = "TabBarSplitterControlRemove";
        public const string TabBarToolTipEmptyIcon = "TabBarToolTipEmptyIcon";
        public const string Test = "Test";
        public const string TextParseFailedFormat = "TextParseFailedFormat";
        public const string Top = "Top";
        public const string TopLeftBottomRight = "TopLeftBottomRight";
        public const string Undo = "Undo";
        public const string OK = "OK";
        public const string Cancel = "Cancel";
        public const string Abort = "Abort";
        public const string Retry = "Retry";
        public const string Ignore = "Ignore";
        public const string Yes = "Yes";
        public const string No = "No";
        public const string Close = "Close";
        public const string Help = "Help";
        public const string TryAgain = "TryAgain";
        public const string Continue = "Continue";
        public const string IgnoreOnce = "IgnoreOnce";
        public const string IgnoreAll = "IgnoreAll";
        public const string Update = "Update";
        public const string Change = "Change";
        public const string ChangeAll = "ChangeAll";
        public const string Options = "Options";
        public const string Add = "Add";
        public const string DeleteAll = "DeleteAll";
        public const string SpellCheckerButtonCustomDictionary = "SpellCheckerButtonCustomDictionary";
        public const string SpellCheckerButtonAddToDictionary = "SpellCheckerButtonAddToDictionary";
        public const string SpellCheckerButtonNew = "SpellCheckerButtonNew";
        public const string SpellCheckerLabelNotInDictionary = "SpellCheckerLabelNotInDictionary";
        public const string SpellCheckerLabelSuggestions = "SpellCheckerLabelSuggestions";
        public const string SpellCheckerLabelOptions = "SpellCheckerLabelOptions";
        public const string SpellCheckerDialogCaption = "SpellCheckerDialogCaption";
        public const string SpellCheckCompletedAlert = "SpellCheckCompletedAlert";
        public const string SpellCheckerDictioanryEditorCaption = "SpellCheckerDictioanryEditorCaption";
        public const string SpellCheckerOptionsDialogCaption = "SpellCheckerOptionsDialogCaption";
        public const string SpellCheckerLabelDictionaryPath = "SpellCheckerLabelDictionaryPath";
        public const string SpellCheckerLabelWords = "SpellCheckerLabelWords";
        public const string SpellCheckerLabelDictionary = "SpellCheckerLabelDictionary";
        public const string SpellCheckerIgnoreUpperCase = "SpellCheckerIgnoreUpperCase";
        public const string SpellCheckerIgnoreMixedCase = "SpellCheckerIgnoreMixedCase";
        public const string SpellCheckerIgnoreWordsWithNumbers = "SpellCheckerIgnoreWordsWithNumbers";
        public const string SpellCheckerIgnoreSpecialCharacters = "SpellCheckerIgnoreSpecialCharacters";
        public const string SpellCheckerIgnoreFileNames = "SpellCheckerIgnoreFileNames";
        public const string SpellCheckerIgnoreEmailAddress = "SpellCheckerIgnoreEmailAddress";
        public const string SpellCheckerIgnoreInternetAddress = "SpellCheckerIgnoreInternetAddress";
        public const string SpellCheckerIgnoreHTMLTags = "SpellCheckerIgnoreHTMLTags";

        #endregion
    }
}