//-------------------------------------------------------------------------------------------------
// <copyright file="GridRichTextEntryPanel.cs" company="syncfusion">
//  Copyright (c) Syncfusion Inc. 2001 - 2006. All rights reserved.
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
// </copyright>
//-------------------------------------------------------------------------------------------------

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Syncfusion.Windows.Forms;
using Syncfusion.Drawing;

namespace Syncfusion.Windows.Forms.Grid
{
    /// <summary>
    /// Provides editing support for RichText. The <see cref="GridRichTextBoxCellRenderer"/>
    /// will display the panel inside a drop-down container.
    /// </summary>
    [ToolboxItem(false)]
    public class GridRichTextEntryPanel : System.Windows.Forms.UserControl
    {
        private System.Windows.Forms.ImageList imageList1;
        private System.Windows.Forms.Panel pnlFonts;
        private System.Windows.Forms.ComboBox fontComboBox;
        private System.Windows.Forms.ComboBox fontSizeComboBox;
        private System.Windows.Forms.Panel pnlFontsTB;
        private System.Windows.Forms.ToolBar tbFont;
        private System.Windows.Forms.ToolBarButton boldButton;
        private System.Windows.Forms.ToolBarButton italicButton;
        private System.Windows.Forms.ToolBarButton underLineButton;
        private System.Windows.Forms.ToolBar tbAlign;
        private System.Windows.Forms.ToolBarButton laButton;
        private System.Windows.Forms.ToolBarButton centerButton;
        private System.Windows.Forms.ToolBarButton raButton;
        private System.Windows.Forms.RichTextBox richTextBox1;
        private System.Windows.Forms.Button btnSave;
        private System.Windows.Forms.Button btnCancel;
        private System.ComponentModel.IContainer components;

        bool initDone = false;

        /// <summary>
        /// Initializes a new <see cref="GridRichTextEntryPanel"/> object.
        /// </summary>
        public GridRichTextEntryPanel()
        {
            // This call is required by the Windows.Forms Form Designer.
            InitializeComponent();

            // TODO: Add any initialization after the InitForm call.
            this.fontComboBox.DisplayMember = "Name";
            this.fontComboBox.ValueMember = "Name";
            this.fontComboBox.Items.AddRange(FontFamily.Families);
            this.fontComboBox.SelectedIndex = 2;
            this.fontSizeComboBox.SelectedIndex = 2;

            this.richTextBox1.TextChanged += new EventHandler(richTextBox1_TextChanged);

            initDone = true;
        }

        /// <summary> 
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }

            base.Dispose(disposing);
        }

        #region Component Designer generated code
        /// <summary> 
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(GridRichTextEntryPanel));
            this.imageList1 = new System.Windows.Forms.ImageList(this.components);
            this.pnlFonts = new System.Windows.Forms.Panel();
            this.fontComboBox = new System.Windows.Forms.ComboBox();
            this.fontSizeComboBox = new System.Windows.Forms.ComboBox();
            this.pnlFontsTB = new System.Windows.Forms.Panel();
            this.tbFont = new System.Windows.Forms.ToolBar();
            this.boldButton = new System.Windows.Forms.ToolBarButton();
            this.italicButton = new System.Windows.Forms.ToolBarButton();
            this.underLineButton = new System.Windows.Forms.ToolBarButton();
            this.tbAlign = new System.Windows.Forms.ToolBar();
            this.laButton = new System.Windows.Forms.ToolBarButton();
            this.centerButton = new System.Windows.Forms.ToolBarButton();
            this.raButton = new System.Windows.Forms.ToolBarButton();
            this.richTextBox1 = new System.Windows.Forms.RichTextBox();
            this.btnSave = new System.Windows.Forms.Button();
            this.btnCancel = new System.Windows.Forms.Button();
            this.pnlFonts.SuspendLayout();
            this.pnlFontsTB.SuspendLayout();
            this.SuspendLayout();
            // 
            // imageList1
            // 
            this.imageList1.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
            this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
            this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
            this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
            // 
            // pnlFonts
            // 
            this.pnlFonts.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                | System.Windows.Forms.AnchorStyles.Right);
            this.pnlFonts.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                                   this.fontComboBox,
                                                                                   this.fontSizeComboBox,
                                                                                   this.pnlFontsTB,
                                                                                   this.tbAlign});
            this.pnlFonts.Name = "pnlFonts";
            this.pnlFonts.Size = new System.Drawing.Size(352, 24);
            this.pnlFonts.TabIndex = 6;
            // 
            // fontComboBox
            // 
            this.fontComboBox.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                | System.Windows.Forms.AnchorStyles.Left)
                | System.Windows.Forms.AnchorStyles.Right);
            this.fontComboBox.DisplayMember = "Name";
            this.fontComboBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.fontComboBox.Location = new System.Drawing.Point(0, 1);
            this.fontComboBox.Name = "fontComboBox";
            this.fontComboBox.Size = new System.Drawing.Size(136, 21);
            this.fontComboBox.TabIndex = 5;
            this.fontComboBox.ValueMember = "Name";
            this.fontComboBox.SelectedIndexChanged += new System.EventHandler(this.fontComboBox_SelectedIndexChanged);
            // 
            // fontSizeComboBox
            // 
            this.fontSizeComboBox.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                | System.Windows.Forms.AnchorStyles.Right);
            this.fontSizeComboBox.Items.AddRange(new object[] {
                                                                  "8",
                                                                  "9",
                                                                  "10",
                                                                  "11",
                                                                  "12",
                                                                  "14",
                                                                  "15"});
            this.fontSizeComboBox.Location = new System.Drawing.Point(140, 1);
            this.fontSizeComboBox.Name = "fontSizeComboBox";
            this.fontSizeComboBox.Size = new System.Drawing.Size(56, 21);
            this.fontSizeComboBox.TabIndex = 4;
            this.fontSizeComboBox.SelectedIndexChanged += new System.EventHandler(this.fontSizeComboBox_SelectedIndexChanged);
            // 
            // pnlFontsTB
            // 
            this.pnlFontsTB.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                | System.Windows.Forms.AnchorStyles.Right);
            this.pnlFontsTB.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                                     this.tbFont});
            this.pnlFontsTB.Location = new System.Drawing.Point(202, 0);
            this.pnlFontsTB.Name = "pnlFontsTB";
            this.pnlFontsTB.Size = new System.Drawing.Size(72, 23);
            this.pnlFontsTB.TabIndex = 5;
            // 
            // tbFont
            // 
            this.tbFont.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                | System.Windows.Forms.AnchorStyles.Right);
            this.tbFont.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
            this.tbFont.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
                                                                                      this.boldButton,
                                                                                      this.italicButton,
                                                                                      this.underLineButton});
            this.tbFont.Divider = false;
            this.tbFont.Dock = System.Windows.Forms.DockStyle.None;
            this.tbFont.DropDownArrows = true;
            this.tbFont.ImageList = this.imageList1;
            this.tbFont.Name = "tbFont";
            this.tbFont.ShowToolTips = true;
            this.tbFont.Size = new System.Drawing.Size(73, 23);
            this.tbFont.TabIndex = 4;
            this.tbFont.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.tbFont_ButtonClick);
            // 
            // boldButton
            // 
            this.boldButton.ImageIndex = 3;
            this.boldButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
            this.boldButton.ToolTipText = "Bold";
            // 
            // italicButton
            // 
            this.italicButton.ImageIndex = 4;
            this.italicButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
            this.italicButton.ToolTipText = "Italic";
            // 
            // underLineButton
            // 
            this.underLineButton.ImageIndex = 5;
            this.underLineButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
            this.underLineButton.ToolTipText = "Underline";
            // 
            // tbAlign
            // 
            this.tbAlign.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                | System.Windows.Forms.AnchorStyles.Right);
            this.tbAlign.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
            this.tbAlign.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
                                                                                       this.laButton,
                                                                                       this.centerButton,
                                                                                       this.raButton});
            this.tbAlign.ButtonSize = new System.Drawing.Size(23, 22);
            this.tbAlign.Divider = false;
            this.tbAlign.Dock = System.Windows.Forms.DockStyle.None;
            this.tbAlign.DropDownArrows = true;
            this.tbAlign.ImageList = this.imageList1;
            this.tbAlign.Location = new System.Drawing.Point(280, 0);
            this.tbAlign.Name = "tbAlign";
            this.tbAlign.ShowToolTips = true;
            this.tbAlign.Size = new System.Drawing.Size(72, 23);
            this.tbAlign.TabIndex = 7;
            this.tbAlign.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.tbAlign_ButtonClick);
            // 
            // laButton
            // 
            this.laButton.ImageIndex = 0;
            this.laButton.Pushed = true;
            this.laButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
            this.laButton.ToolTipText = "Left Aligned";
            // 
            // centerButton
            // 
            this.centerButton.ImageIndex = 1;
            this.centerButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
            this.centerButton.ToolTipText = "Center Aligned";
            // 
            // raButton
            // 
            this.raButton.ImageIndex = 2;
            this.raButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
            this.raButton.ToolTipText = "Right Aligned";
            // 
            // richTextBox1
            // 
            this.richTextBox1.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                | System.Windows.Forms.AnchorStyles.Left)
                | System.Windows.Forms.AnchorStyles.Right);
            this.richTextBox1.Location = new System.Drawing.Point(0, 24);
            this.richTextBox1.Name = "richTextBox1";
            this.richTextBox1.Size = new System.Drawing.Size(352, 168);
            this.richTextBox1.TabIndex = 7;
            this.richTextBox1.Text = string.Empty;
            this.richTextBox1.SelectionChanged += new System.EventHandler(this.richTextBox1_SelectionChanged);
            // 
            // btnSave
            // 
            this.btnSave.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
            this.btnSave.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.btnSave.FlatStyle = System.Windows.Forms.FlatStyle.System;
            this.btnSave.Location = new System.Drawing.Point(176, 200);
            this.btnSave.Name = "btnSave";
            this.btnSave.Size = new System.Drawing.Size(72, 24);
            this.btnSave.TabIndex = 8;
            this.btnSave.Text = "&Save";
            this.btnSave.Click += new System.EventHandler(this.btnSave_Click);
            // 
            // btnCancel
            // 
            this.btnCancel.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
            this.btnCancel.CausesValidation = false;
            this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
            this.btnCancel.Location = new System.Drawing.Point(264, 200);
            this.btnCancel.Name = "btnCancel";
            this.btnCancel.Size = new System.Drawing.Size(72, 24);
            this.btnCancel.TabIndex = 8;
            this.btnCancel.Text = "&Cancel";
            this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
            // 
            // GridRichTextEntryPanel
            // 
            this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                          this.btnSave,
                                                                          this.richTextBox1,
                                                                          this.pnlFonts,
                                                                          this.btnCancel});
            this.Name = "GridRichTextEntryPanel";
            this.Size = new System.Drawing.Size(352, 232);
            this.pnlFonts.ResumeLayout(false);
            this.pnlFontsTB.ResumeLayout(false);
            this.ResumeLayout(false);

        }
        #endregion

        private FontStyle curStyle = FontStyle.Regular;
        private HorizontalAlignment curAlignment = HorizontalAlignment.Left;

        private void tbFont_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
        {
            if (!initDone)
            {
                return;
            }

            tbAlign_ButtonClick(sender, e);
        }

        private void tbAlign_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
        {
            if (!initDone)
            {
                return;
            }

            if (e.Button == this.boldButton)
            {
                if (this.boldButton.Pushed)
                {
                    this.curStyle = this.curStyle | FontStyle.Bold;
                }
                else
                {
                    this.curStyle &= ~FontStyle.Bold;
                }
            }
            else if (e.Button == this.italicButton)
            {
                if (this.italicButton.Pushed)
                {
                    this.curStyle = this.curStyle | FontStyle.Italic;
                }
                else
                {
                    this.curStyle &= ~FontStyle.Italic;
                }
            }
            else if (e.Button == this.underLineButton)
            {
                if (this.underLineButton.Pushed)
                {
                    this.curStyle = this.curStyle | FontStyle.Underline;
                }
                else
                {
                    this.curStyle &= ~FontStyle.Underline;
                }
            }
            else if (e.Button == this.laButton)
            {
                if (this.laButton.Pushed)
                {
                    this.curAlignment = HorizontalAlignment.Left;
                    this.centerButton.Pushed = false;
                    this.raButton.Pushed = false;
                }

                this.richTextBox1.SelectionAlignment = this.curAlignment;
            }
            else if (e.Button == this.centerButton)
            {
                if (this.centerButton.Pushed)
                {
                    this.curAlignment = HorizontalAlignment.Center;
                    this.laButton.Pushed = false;
                    this.raButton.Pushed = false;
                }

                this.richTextBox1.SelectionAlignment = this.curAlignment;
            }
            else if (e.Button == this.raButton)
            {
                if (this.raButton.Pushed)
                {
                    this.curAlignment = HorizontalAlignment.Right;
                    this.laButton.Pushed = false;
                    this.centerButton.Pushed = false;
                }

                this.richTextBox1.SelectionAlignment = this.curAlignment;
            }

            this.richTextBox1.Focus();
            this.UpdateFont();
        }

        private void fontComboBox_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            if (!initDone)
            {
                return;
            }

            this.richTextBox1.Focus();
            this.UpdateFont();
        }

        private void fontSizeComboBox_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            if (!initDone)
            {
                return;
            }

            this.richTextBox1.Focus();
            this.UpdateFont();
        }

        private void UpdateFont()
        {
            try
            {
                string family = this.fontComboBox.SelectedItem.ToString();
                int n2 = family.IndexOf("]");
                int n3 = family.IndexOf("=");
                if (n2 != -1 && n2 > n3)
                {
                    family = family.Substring(family.IndexOf("=") + 1, family.IndexOf("]") - family.IndexOf("=") - 1);
                }

                this.richTextBox1.SelectionFont = FontUtil.CreateFont(family, float.Parse(this.fontSizeComboBox.Text), this.curStyle);
            }
            catch 
            { 
            }
        }

        private void btnSave_Click(object sender, System.EventArgs e)
        {
            if (Save != null)
            {
                Save(this, EventArgs.Empty);
            }
        }

        private void btnCancel_Click(object sender, System.EventArgs e)
        {
            if (Cancel != null)
            {
                Cancel(this, EventArgs.Empty);
            }
        }

        private void richTextBox1_SelectionChanged(object sender, System.EventArgs e)
        {
            if (!initDone)
            {
                return;
            }

            initDone = false;
            Font f = richTextBox1.SelectionFont;

            if (f != null)
            {
                try
                {
                    curStyle = f.Style;

                    this.fontComboBox.SelectedIndex = this.fontComboBox.FindString(f.Name);
                    this.fontSizeComboBox.Text = f.SizeInPoints.ToString();
                    this.boldButton.Pushed = (f.Style & FontStyle.Bold) != 0;
                    this.underLineButton.Pushed = (f.Style & FontStyle.Underline) != 0;
                    this.italicButton.Pushed = (f.Style & FontStyle.Italic) != 0;

                    curAlignment = this.richTextBox1.SelectionAlignment;
                    this.laButton.Pushed = curAlignment == HorizontalAlignment.Left;
                    this.centerButton.Pushed = curAlignment == HorizontalAlignment.Center;
                    this.raButton.Pushed = curAlignment == HorizontalAlignment.Right;
                }
                catch
                {
                }
            }
            // TODO: What about text color?
            initDone = true;
        }

        /// <returns>
        /// true if the key was processed by the control; otherwise, false.
        /// </returns>
        /// <override/>
        protected override bool ProcessDialogKey(Keys keyData)
        {
            ////Syncfusion.Diagnostics.TraceUtil.TraceCurrentMethodInfo(keyData);
            if (this.ActiveControl == this.richTextBox1)
            {
                if (keyData == Keys.Enter)
                {
                    return false;
                }
            }

            return base.ProcessDialogKey(keyData);
        }

        /// <summary>
        /// Occurs when user clicks the "Save" button.
        /// </summary>
        public event EventHandler Save;

        /// <summary>
        /// Occurs when user clicks the "Cancel" button.
        /// </summary>
        public event EventHandler Cancel;

        /// <summary>
        /// Gets a reference to the <see cref="RichTextBox"/> that is being displayed.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public System.Windows.Forms.RichTextBox RichTextBox
        {
            get
            {
                return this.richTextBox1;
            }
        }

        private void richTextBox1_TextChanged(object sender, EventArgs e)
        {
            richTextBox1_SelectionChanged(sender, e);
            this.richTextBox1.Font = Font;
        }
    }
}
