#Region "Copyright Syncfusion Inc. 2001 - 2006"
'
'  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
'
'  Use of this code is subject to the terms of our license.
'  A copy of the current license can be obtained at any time by e-mailing
'  licensing@syncfusion.com. Any infringement will be prosecuted under
'  applicable laws. 
'
#End Region

Imports System
Imports System.Drawing
Imports System.Windows.Forms

Imports Syncfusion.Styles
Imports Syncfusion.Windows.Forms.Grid
Imports Syncfusion.Windows.Forms.InternalMenus



'/ <summary>
'/ This class is the common base class for actions such as
'/ MenuActions, Editactions or plugins
'/ </summary>

Public MustInherit Class GridPadBasicAction
    Inherits BasicAction
    Private window As MainWindow = Nothing


    Public ReadOnly Property ActiveGrid() As GridControlBase
        Get
            If Not (MainWindow Is Nothing) Then
                If TypeOf MainWindow.ActiveControl Is Form Then
                    Return GetGrid(CType(MainWindow.ActiveControl, Form))
                ElseIf Not (MainWindow.ActiveControl Is Nothing) Then
                    Return GetGrid(CType(MainWindow.ActiveControl.FindForm(), Form))
                End If
            End If
            Return Nothing
        End Get
    End Property


    Public Function GetGrid(ByVal form As Form) As GridControlBase
        If Not (form Is Nothing) Then
            Dim control As control = form.ActiveControl
            If Not (control Is Nothing) Then
                While TypeOf control Is ContainerControl
                    control = CType(control, ContainerControl).ActiveControl
                End While
                ' Could be a focused textbox
                If Not TypeOf control Is GridControlBase Then
                    control = control.Parent
                End If
                If TypeOf control Is GridControlBase Then
                    Dim grid As GridControlBase = CType(control, GridControlBase)

                    ' Active cell ...
                    If Not (control Is Nothing) AndAlso grid Is Nothing Then
                        grid = IIf(TypeOf control.Parent Is GridControlBase, CType(grid, GridControlBase), Nothing)
                    End If
                    Return grid
                End If
            End If
        End If
        Return Nothing
    End Function 'GetGrid


    Public Sub ApplyStyle(ByVal style As GridStyleInfo)
        Dim grid As GridControlBase = ActiveGrid
        If Not (grid Is Nothing) Then
            Dim ranges As GridRangeInfoList = New GridRangeInfoList
            grid.Selections.GetSelectedRanges(ranges, True)
            Dim range As GridRangeInfo
            For Each range In ranges
                grid.Model.ChangeCells(range, style, StyleModifyType.Override)
            Next range
        End If
    End Sub 'ApplyStyle 

    Public Overloads Property MainWindow() As MainWindow
        Get
            Return MyBase.MainWindow
        End Get
        Set(ByVal Value As MainWindow)
            MyBase.MainWindow = Value
        End Set
    End Property
End Class 'BasicAction
