#Region "Copyright Syncfusion Inc. 2001 - 2006"
'
'  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
'
'  Use of this code is subject to the terms of our license.
'  A copy of the current license can be obtained at any time by e-mailing
'  licensing@syncfusion.com. Any infringement will be prosecuted under
'  applicable laws. 
'
#End Region

Imports System
Imports System.Drawing
Imports System.Drawing.Printing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.IO
Imports System.Threading
Imports System.Diagnostics

Imports Syncfusion.Samples.GridPad
Imports Syncfusion.Styles
Imports Syncfusion.Windows.Forms
Imports Syncfusion.Windows.Forms.Grid
Imports Syncfusion.Drawing


Namespace Actions

    Public Class ExitProgram
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            MainWindow.Close()
        End Sub 'InvokeAction
    End Class 'ExitProgram



    Public Class Find
        Inherits GridPadBasicAction
        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As SampleGrid = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim frDialog As GridFindReplaceDialog = GridFindReplaceDialog.Instance
                frDialog.SetState(grid.GridFindReplaceDialogSink, "", False)
                frDialog.Show()
            End If
        End Sub 'InvokeAction
    End Class 'Find


    Public Class FindNext
        Inherits GridPadBasicAction
        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As SampleGrid = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim frDialog As GridFindReplaceDialog = GridFindReplaceDialog.Instance
                frDialog.FindNext()
            End If
        End Sub 'InvokeAction
    End Class 'FindNext


    Public Class Replace
        Inherits GridPadBasicAction
        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As SampleGrid = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim frDialog As GridFindReplaceDialog = GridFindReplaceDialog.Instance
                frDialog.SetState(grid.GridFindReplaceDialogSink, "", True)
                frDialog.Show()
            End If
        End Sub 'InvokeAction
    End Class 'Replace
    Public Class PrintPreviewFile
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Try
                    Dim pd As New GridPrintDocument(grid, True) 'Assumes the default printer

                    If Not (PrinterSettings.storedPageSettings Is Nothing) Then
                        pd.DefaultPageSettings = PrinterSettings.storedPageSettings
                    End If

                    Dim dlg As New PrintPreviewDialog
                    dlg.Document = pd
                    dlg.ShowDialog()
                Catch ex As Exception
                    MessageBox.Show(("An error occurred attempting to preview the file to print - " + ex.Message))
                End Try
            End If
        End Sub 'InvokeAction
    End Class 'PrintPreviewFile


    Public Class PrintFile
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Try
                    Dim pd As New GridPrintDocument(grid) 'Assumes the default printer
                    If Not (PrinterSettings.storedPageSettings Is Nothing) Then
                        pd.DefaultPageSettings = PrinterSettings.storedPageSettings
                    End If

                    Dim dlg As New PrintDialog
                    dlg.Document = pd
                    dlg.AllowSelection = True
                    dlg.AllowSomePages = True
                    Dim result As DialogResult = dlg.ShowDialog()

                    If result = DialogResult.OK Then
                        pd.Print()
                    End If
                Catch ex As Exception
                    MessageBox.Show(("An error occurred attempting to preview the file to print - " + ex.Message))
                End Try
            End If
        End Sub 'InvokeAction
    End Class 'PrintFile


    Public Class PrinterSettings
        Inherits GridPadBasicAction
        Public Shared storedPageSettings As PageSettings = Nothing


        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Try
                Dim psDlg As New PageSetupDialog

                If storedPageSettings Is Nothing Then
                    storedPageSettings = New PageSettings
                End If

                psDlg.PageSettings = storedPageSettings
                psDlg.ShowDialog()

            Catch ex As Exception
                MessageBox.Show(("An error occurred - " + ex.Message))
            End Try
        End Sub 'InvokeAction
    End Class 'PrinterSettings



    Public Class BlackWhiteGrid
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.Properties.BlackWhite = Not grid.Model.Properties.BlackWhite
                grid.Model.Refresh()
            End If
        End Sub 'InvokeAction
    End Class 'BlackWhiteGrid


    Public Class AlphaBlending
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.SupportsTransparentBackColor = Not grid.SupportsTransparentBackColor
                If grid.SupportsTransparentBackColor Then
                    grid.Model.TableStyle.Interior = New BrushInfo(Color.FromArgb(50, Color.White))
                Else
                    grid.Model.TableStyle.ResetInterior()
                End If
                grid.Model.Options.TransparentBackground = Not grid.SupportsTransparentBackColor AndAlso grid.BackgroundImage Is Nothing
            End If
        End Sub 'InvokeAction
    End Class 'AlphaBlending


    Public Class TransparentBackground
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.Options.TransparentBackground = Not grid.Model.Options.TransparentBackground
            End If
        End Sub 'InvokeAction
    End Class 'TransparentBackground


    Public Class ResetTransparency
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.Options.TransparentBackground = False
                grid.SupportsTransparentBackColor = False
                grid.BackgroundImage = Nothing
                grid.BackColor = Color.White
            End If
        End Sub 'InvokeAction
    End Class 'ResetTransparency


    Public Class BackgroundImage
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim c As Control = grid
                ' does not work for SplitterControl right now
                If TypeOf grid.Parent Is SplitterControl Then
                    c = grid.Parent
                End If
                If c.BackgroundImage Is Nothing Then
                    ' (grid.TransparentBackground completely disables background drawing for all cells
                    ' whereas SupportsTransparentBackColor = true  together with TransparentBackground
                    ' lets me do alphablending).
                    Dim type As type = GetType(Bitmap)
                    Dim stream As System.IO.Stream = GetType(SampleGrid).Module.Assembly.GetManifestResourceStream((ApplicationMain.manifestNamespace + "logo_800_edf0f7.jpg"))
                    Dim bm As New Bitmap(stream)
                    c.BackgroundImage = bm
                Else
                    c.BackgroundImage = Nothing
                End If
            End If
        End Sub 'InvokeAction
    End Class 'BackgroundImage




    Public Class FormatBaseStylesMap
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                GridBaseStylesMap.ShowGridBaseStylesMapDialog(grid.Model, "BaseStylesMap")
                grid.Model.Refresh()
            End If
        End Sub 'InvokeAction
    End Class 'FormatBaseStylesMap


    Public Class FormatOptions
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim form As New PropertyForm(grid.Model.Options)
                form.ShowDialog()
                grid.Model.Refresh()
            End If
        End Sub 'InvokeAction
    End Class 'FormatOptions


    Public Class FormatProperties
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim form As New PropertyForm(grid.Model.Properties)
                form.ShowDialog()
                grid.Model.Refresh()
            End If
        End Sub 'InvokeAction
    End Class 'FormatProperties


    Public Class FormatCells
        Inherits GridPadBasicAction
        Private style As GridStyleInfo = Nothing
        Private ranges As GridRangeInfoList = Nothing
        Private grid As GridControlBase = Nothing


        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            grid = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Selections.GetSelectedRanges(ranges, True)
                GetCombinedStyle()

                If Not (style Is Nothing) Then
                    Dim form As New StyleInfoForm(style)
                    AddHandler form.ApplyButton.Click, AddressOf Me.applyButton_Click
                    form.Text = "Cells"

                    If form.ShowDialog(MainWindow) = DialogResult.OK Then
                        Trace.WriteLine(style.ToString("d"))
                        ApplyChanges()
                    End If
                End If
            End If
        End Sub 'InvokeAction


        Protected Sub applyButton_Click(ByVal sender As Object, ByVal e As System.EventArgs)
            ApplyChanges()
            GetCombinedStyle()
        End Sub 'applyButton_Click


        Public Sub GetCombinedStyle()
            style = grid.Model.GetCombinedStyle(ranges)
        End Sub 'GetCombinedStyle


        Public Sub ApplyChanges()
            If Not (grid Is Nothing) Then
                Dim range As GridRangeInfo
                For Each range In ranges
                    grid.Model.ChangeCells(range, style, StyleModifyType.Changes)
                Next range
            End If
        End Sub 'ApplyChanges
    End Class 'FormatCells


    Public Class FontStyleBold
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim style As New GridStyleInfo
            style.Font.Bold = True
            ApplyStyle(style)

            'Dim c As ContainerControl = MainWindow
            'While Not (c Is Nothing)
            '    If TypeOf c Is SplitterControl Then
            '        CType(c, SplitterControl).ScrollBarColor = Color.Red
            '        Exit While
            '    End If
            '    If Not (c.ActiveControl Is Nothing) And TypeOf c.ActiveControl Is ContainerControl Then
            '        c = c.ActiveControl
            '    Else
            '        c = Nothing
            '    End If
            'End While
        End Sub 'InvokeAction
    End Class 'FontStyleBold


    Public Class FontStyleItalic
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim style As New GridStyleInfo
            style.Font.Italic = True
            ApplyStyle(style)
        End Sub 'InvokeAction
    End Class 'FontStyleItalic


    Public Class FontStyleUnderline
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim style As New GridStyleInfo
            style.Font.Underline = True
            ApplyStyle(style)
        End Sub 'InvokeAction
    End Class 'FontStyleUnderline


    Public Class AlignLeft
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim style As New GridStyleInfo
            style.HorizontalAlignment = GridHorizontalAlignment.Left
            ApplyStyle(style)
        End Sub 'InvokeAction
    End Class 'AlignLeft


    Public Class AlignCenter
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim style As New GridStyleInfo
            style.HorizontalAlignment = GridHorizontalAlignment.Center
            ApplyStyle(style)
        End Sub 'InvokeAction
    End Class 'AlignCenter


    Public Class AlignRight
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim style As New GridStyleInfo
            style.HorizontalAlignment = GridHorizontalAlignment.Right
            ApplyStyle(style)
        End Sub 'InvokeAction
    End Class 'AlignRight


    Public Class ResizeColumns
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim range As GridRangeInfo
                For Each range In grid.Model.Selections
                    grid.Model.ColWidths.ResizeToFit(range, GridResizeToFitOptions.ResizeCoveredCells)
                Next range
            End If
        End Sub 'InvokeAction
    End Class 'ResizeColumns


    Public Class ResizeRows
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim range As GridRangeInfo
                For Each range In grid.Model.Selections
                    grid.Model.RowHeights.ResizeToFit(range, GridResizeToFitOptions.ResizeCoveredCells)
                Next range
            End If
        End Sub 'InvokeAction
    End Class 'ResizeRows


    Public Class RemoveRows
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim ranges As GridRangeInfoList = grid.Model.Selections.Ranges.GetRowRanges(GridRangeInfoType.Rows)
                If ranges.Count = 0 Then
                    Return
                End If
                grid.Model.CommandStack.BeginTrans("Remove Rows")

                Dim range As GridRangeInfo
                For Each range In ranges
                    grid.Model.Rows.RemoveRange(range.Top, range.Bottom)
                Next range
                grid.Model.CommandStack.CommitTrans()
            End If
        End Sub 'InvokeAction
    End Class 'RemoveRows


    Public Class RemoveCols
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim ranges As GridRangeInfoList = grid.Model.Selections.Ranges.GetColRanges(GridRangeInfoType.Cols)
                If ranges.Count = 0 Then
                    Return
                End If
                grid.Model.CommandStack.BeginTrans("Remove Columns")

                Dim range As GridRangeInfo
                For Each range In ranges
                    grid.Model.Cols.RemoveRange(range.Left, range.Right)
                Next range
                grid.Model.CommandStack.CommitTrans()
            End If
        End Sub 'InvokeAction
    End Class 'RemoveCols


    Public Class InsertRows
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim rowIndex, colIndex As Integer
                If grid.CurrentCell.GetCurrentCell(rowIndex, colIndex) Then
                    grid.Model.Rows.InsertRange(rowIndex, 1, Nothing)
                End If
            End If
        End Sub 'InvokeAction
    End Class 'InsertRows


    Public Class InsertCols
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim rowIndex, colIndex As Integer
                If grid.CurrentCell.GetCurrentCell(rowIndex, colIndex) Then
                    grid.Model.Cols.InsertRange(colIndex, 1, Nothing)
                End If
            End If
        End Sub 'InvokeAction
    End Class 'InsertCols


    Public Class CoverCell
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim range As GridRangeInfo = grid.Model.Selections.Ranges.ActiveRange
                grid.Model.CoveredRanges.Add(range)
            End If
        End Sub 'InvokeAction
    End Class 'CoverCell


    Public Class ResetCover
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim rl As GridRangeInfoList = New GridRangeInfoList
                If grid.Selections.GetSelectedRanges(rl, True) Then
                    Dim range As GridRangeInfo = rl.ActiveRange
                    If Not range.IsEmpty Then
                        range = grid.Model.CoveredRanges.Merge(range)
                    End If
                    grid.Model.CoveredRanges.Remove(range)
                End If
            End If
        End Sub 'InvokeAction

        Public Sub New()

        End Sub
    End Class 'ResetCover


    Public Class FreezeColumns
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.Cols.FreezeSelection()
            End If
        End Sub 'InvokeAction
    End Class 'FreezeColumns 

    Public Class ReleaseColumns
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.Cols.RestoreFrozen()
            End If
        End Sub 'InvokeAction
    End Class 'ReleaseColumns 

    Public Class FreezeRows
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.Rows.FreezeSelection()
            End If
        End Sub 'InvokeAction
    End Class 'FreezeRows 

    Public Class ReleaseRows
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.Rows.RestoreFrozen()
            End If
        End Sub 'InvokeAction
    End Class 'ReleaseRows 




    Public Class NewFile
        Inherits GridPadBasicAction
        Private windowCount As Integer = 0

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            windowCount += 1
            Dim doc As New GridFrame
            doc.Text = "Document " + windowCount.ToString()
            doc.MdiParent = MainWindow

            doc.Show()

            Dim ba As GridPadBasicAction = New Syncfusion.Samples.GridPad.Actions.AlphaBlending
            ba.MainWindow = MainWindow
            ba.InvokeAction(sender, e)

            ba = New Syncfusion.Samples.GridPad.Actions.BackgroundImage
            ba.MainWindow = MainWindow
            ba.InvokeAction(sender, e)
        End Sub 'InvokeAction 
    End Class 'NewFile

    Public Class NewDialog
        Inherits GridPadBasicAction
        Private windowCount As Integer = 0

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            windowCount += 1
            Dim doc As New GridDialog
            doc.Text = "Document " + Str$(windowCount)
            doc.MdiParent = MainWindow
            doc.Show()
        End Sub 'InvokeAction 
    End Class 'NewDialog


    Public Class NewWorkbookFile
        Inherits GridPadBasicAction
        Private windowCount As Integer = 0
        Private workbook As WorkbookModel

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            windowCount += 1
            workbook = New WorkbookModel("Workbook")
            Dim sheet1 As New SampleGridModel
            SampleGrid.SetupGridModel(sheet1)
            Dim sheet2 As New SampleGridModel
            SampleGrid.SetupGridModel(sheet2)

            workbook.Worksheets.Add(New WorksheetModel(workbook, "Sheet 1", sheet1))
            workbook.Worksheets.Add(New WorksheetModel(workbook, "Sheet 2", sheet2))

            Dim doc As New WorkbookForm(workbook)
            doc.Text = workbook.Name + Str$(windowCount)
            doc.MdiParent = MainWindow
            doc.ThemesEnabled = True
            doc.Show()
        End Sub 'InvokeAction
    End Class 'NewWorkbookFile


    Public Class NewSplitterFile
        Inherits GridPadBasicAction
        Private windowCount As Integer = 0

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            windowCount += 1
            Dim doc As New SplitterFrame
            doc.Text = "Splitter " + Str$(windowCount)
            doc.MdiParent = MainWindow
            doc.Show()
        End Sub 'InvokeAction
    End Class 'NewSplitterFile


    Public Class NewRecordFile
        Inherits GridPadBasicAction
        Private windowCount As Integer = 0

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            windowCount += 1
            Dim doc As New RecordFrame
            doc.Text = "Record " + Str$(windowCount)
            doc.MdiParent = MainWindow
            doc.Show()
        End Sub 'InvokeAction
    End Class 'NewRecordFile



    Public Class OpenFile
        Inherits GridPadBasicAction


        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim fdiag As New OpenFileDialog
            fdiag.AddExtension = True
            fdiag.Filter = "EssentialGrid files (*.obg)|*.obg"
            fdiag.Multiselect = True
            fdiag.CheckFileExists = True

            If fdiag.ShowDialog() = DialogResult.OK Then
                Dim name As [String]
                For Each name In fdiag.FileNames
                    Dim doc As New GridFrame(GridModel.LoadBinary(name))
                    doc.MdiParent = MainWindow
                    doc.Text = name
                    doc.Show()
                Next name 'Options.AddLastFile(name);
            End If
            'MainWindow.UpdateLastFiles();
            fdiag.Dispose()
        End Sub 'InvokeAction
    End Class 'OpenFile


    Public Class CloseFile
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim form As form = IIf(TypeOf MainWindow.ActiveControl Is form, CType(MainWindow.ActiveControl, form), Nothing)
            If Not (form Is Nothing) Then
                form.Close()
            End If
        End Sub 'InvokeAction
    End Class 'CloseFile 

    Public Class CloseAll
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim form As form
            For Each form In MainWindow.MdiChildren
                form.Close()
            Next form
        End Sub 'InvokeAction
    End Class 'CloseAll


    Public Class SaveFile
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim model As GridModel = grid.Model
                If model.IsUntitled Then
                    Dim sfa As New SaveFileAs
                    sfa.MainWindow = MainWindow
                    sfa.InvokeAction(sender, e)
                Else
                    model.SaveBinary()
                End If
            End If
        End Sub 'InvokeAction
    End Class 'SaveFile 


    Public Class SaveFileAs
        Inherits GridPadBasicAction
        Private fdiag As SaveFileDialog = Nothing

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                Dim model As GridModel = grid.Model
                If fdiag Is Nothing Then
                    fdiag = New SaveFileDialog
                    fdiag.OverwritePrompt = True
                    fdiag.AddExtension = True
                    fdiag.Filter = "EssentialGrid files (*.obg)|*.obg"
                End If

                If fdiag.ShowDialog() = DialogResult.OK Then
                    model.SaveBinary(fdiag.FileName)
                    MessageBox.Show(fdiag.FileName, "File saved", MessageBoxButtons.OK)
                    grid.Parent.Text = fdiag.FileName
                End If
            End If
        End Sub 'InvokeAction
    End Class 'SaveFileAs


    Public Class SaveAll
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim form As form
            For Each form In MainWindow.MdiChildren
                Dim grid As GridControlBase = GetGrid(form)
                If Not (grid Is Nothing) Then
                    Dim model As GridModel = grid.Model
                    If model.IsUntitled Then
                        Dim sfa As New SaveFileAs
                        sfa.MainWindow = MainWindow
                        sfa.InvokeAction(sender, e)
                    Else
                        model.SaveBinary()
                    End If
                End If
            Next form
        End Sub 'InvokeAction
    End Class 'SaveAll 
    '//////////////// Edit Menu

    Public Class UndoAction
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.CommandStack.Undo()
            End If
        End Sub 'InvokeAction
    End Class 'UndoAction 

    Public Class RedoAction
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.CommandStack.Redo()
            End If
        End Sub 'InvokeAction
    End Class 'RedoAction 

    Public Class CutAction
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.CutPaste.Cut()
            End If
        End Sub 'InvokeAction
    End Class 'CutAction 

    Public Class CopyAction
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.CutPaste.Copy()
            End If
        End Sub 'InvokeAction
    End Class 'CopyAction 

    Public Class PasteAction
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.CutPaste.Paste()
            End If
        End Sub 'InvokeAction
    End Class 'PasteAction 

    Public Class DeleteAction
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.Clear(True)
            End If
        End Sub 'InvokeAction
    End Class 'DeleteAction 

    Public Class SelectAllAction
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Selections.Clear()
                grid.Selections.Add(GridRangeInfo.Table())
            End If
        End Sub 'InvokeAction
    End Class 'SelectAllAction


    Public Class ExcelLikeSelection
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim grid As GridControlBase = ActiveGrid
            If Not (grid Is Nothing) Then
                grid.Model.BeginUpdate(BeginUpdateOptions.None)
                Dim bSelect As Boolean = Not grid.Model.Options.ExcelLikeCurrentCell
                grid.Model.Options.ExcelLikeCurrentCell = bSelect
                grid.Model.Options.ExcelLikeSelectionFrame = bSelect
                Dim range As GridRangeInfo = grid.Selections.Ranges.ActiveRange
                If Not range.IsEmpty Then
                    grid.CurrentCell.ConfirmChanges()
                    grid.CurrentCell.Deactivate(True)
                    grid.CurrentCell.MoveTo(range.Top, range.Left, GridSetCurrentCellOptions.NoSelectRange)
                End If
                grid.Model.EndUpdate(False)
                grid.Model.Refresh()
            End If
        End Sub 'InvokeAction
    End Class 'ExcelLikeSelection


    '/////// Window Menu

    Public Class WindowNew
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            If Not (MainWindow Is Nothing) Then
                If (TypeOf MainWindow.ActiveControl Is WorkbookForm) Then
                    Dim form As WorkbookForm
                    form = CType(MainWindow.ActiveControl, WorkbookForm)
                    form = New WorkbookForm(form.WorkbookView.Workbook)
                    form.MdiParent = MainWindow
                    form.Show()
                    Return
                Else
                    Dim grid As GridControlBase = ActiveGrid
                    If Not (grid Is Nothing) Then
                        Dim model As GridModel = grid.Model
                        Dim doc As New GridFrame(model)
                        doc.Text = CType(MainWindow.ActiveControl, form).Text + ":2"
                        doc.MdiParent = MainWindow
                        doc.Show()
                    End If
                End If
            End If
        End Sub 'InvokeAction
    End Class 'WindowNew


    Public Class WindowCascade
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            MainWindow.LayoutMdi(MdiLayout.Cascade)
        End Sub 'InvokeAction
    End Class 'WindowCascade


    Public Class WindowTileHorizontal
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            MainWindow.LayoutMdi(MdiLayout.TileHorizontal)
        End Sub 'InvokeAction
    End Class 'WindowTileHorizontal


    Public Class WindowTileVertical
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            MainWindow.LayoutMdi(MdiLayout.TileVertical)
        End Sub 'InvokeAction
    End Class 'WindowTileVertical


    Public Class WindowArrangeIcons
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            MainWindow.LayoutMdi(MdiLayout.ArrangeIcons)
        End Sub 'InvokeAction
    End Class 'WindowArrangeIcons
    Public Class AboutObjectGrid
        Inherits GridPadBasicAction

        Public Overrides Sub InvokeAction(ByVal sender As Object, ByVal e As EventArgs)
            Dim abtForm As DemoCommon.AboutForm = New DemoCommon.AboutForm(AppDomain.CurrentDomain.GetAssemblies())
            abtForm.ShowDialog()
        End Sub 'InvokeAction
    End Class 'AboutObjectGrid
End Namespace 'Syncfusion.Samples.GridPad.MenuAction 
