#Region "Copyright Syncfusion Inc. 2001 - 2006"
'
'  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
'
'  Use of this code is subject to the terms of our license.
'  A copy of the current license can be obtained at any time by e-mailing
'  licensing@syncfusion.com. Any infringement will be prosecuted under
'  applicable laws. 
'
#End Region

Imports System
Imports System.Collections
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports System.Data
Imports System.Diagnostics
Imports System.Drawing
Imports System.Globalization
Imports System.Threading
Imports System.Windows.Forms

Imports Syncfusion.ComponentModel
Imports Syncfusion.Drawing
Imports Syncfusion.Styles
Imports Syncfusion.Windows.Forms
Imports Syncfusion.Windows.Forms.Grid
Imports System.Runtime.Serialization

'/ <summary>
'/ SampleGridModel for Workbook/Worksheet support (see MenuAction.NewWorkbookFile)
'/ </summary>

Public Class SampleGridModel
    Inherits GridModel
    Public Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        MyBase.New(info, context)

    End Sub
    Public Sub New()
        MyBase.New()

    End Sub
    Overrides Function CreateControl() As Control
        Dim grid As SampleGrid = New SampleGrid(Me)
        grid.FillSplitterPane = True
        Return grid
    End Function 'ICreateControl.CreateControl
End Class 'SampleGridModel

'/ <summary>
'/    A derived grid component class.
'/ </summary>

Public Class SampleGrid
    Inherits GridControl

    Public Sub New()
        MyClass.New(Nothing)
    End Sub 'New


    Public Sub New(ByVal model As GridModel)
        MyBase.New(model)
        Me.FillSplitterPane = True

        ' transparent
        Dim alphablending As Boolean = False
        If alphablending Then
            Me.SupportsTransparentBackColor = True
            Me.BackColor = Color.FromArgb(99, Color.White)

            Dim type As type = GetType(Bitmap)
            Dim stream As System.IO.Stream = [GetType]().Module.Assembly.GetManifestResourceStream((ApplicationMain.manifestNamespace + "logo_800_edf0f7.jpg"))
            Dim bm As New Bitmap(stream)
            Me.BackgroundImage = bm
        Else
            Me.BackColor = Color.White
            Me.ForeColor = SystemColors.WindowText
        End If
        Me.VerticalThumbTrack = False
        Me.VerticalScrollTips = True
        Me.HorizontalThumbTrack = True
        Me.HorizontalScrollTips = True
        Me.ThemesEnabled = True
        Me.GridVisualStyles = Syncfusion.Windows.Forms.GridVisualStyles.Metro
        Me.Properties.GridLineColor = Color.FromArgb(208, 215, 229)
        Me.TableStyle.TextColor = Color.MidnightBlue
        Me.TableStyle.Font.Facename = "Verdana"
        Me.DefaultGridBorderStyle = GridBorderStyle.Solid
    End Sub 'New

    ' Find and replace dialog
    Dim findReplaceDialogSink As GridFindReplaceDialogSink


    Public ReadOnly Property GridFindReplaceDialogSink() As GridFindReplaceDialogSink
        Get
            If findReplaceDialogSink Is Nothing Then
                findReplaceDialogSink = New GridFindReplaceDialogSink(Me)
            End If
            Return findReplaceDialogSink
        End Get
    End Property


    Protected Overrides Sub OnCurrentCellActivated(ByVal e As EventArgs)
        GridFindReplaceDialog.ResetFindLocation()
        MyBase.OnCurrentCellActivated(e)
    End Sub 'OnCurrentCellActivated

    Protected Overrides Sub OnControlGotFocus()
        GridFindReplaceDialog.SetActiveSinkIfVisible(GridFindReplaceDialogSink)
        MyBase.OnControlGotFocus()
    End Sub 'OnEnter

    Public Overrides Sub Initialize()
        MyBase.Initialize()
        Me.TopRowIndex = InternalGetHeaderRows() + 1
        Me.LeftColIndex = InternalGetHeaderCols() + 1
        Me.AllowDrop = True
    End Sub 'Initialize



    '/ <summary>
    '/ This event handler demonstrates how to handle the QueryNextCurrentCellPosition
    '/ event in order to skip not only disabled cells but also skip readonly cells.
    '/ </summary>
    '/ <param name="e">Event data.</param>
    Protected Overrides Sub OnQueryNextCurrentCellPosition(ByVal e As GridQueryNextCurrentCellPositionEventArgs)
        ' just set this flag true if you want to try out this event handler.
        Dim flag As Boolean = False
        If flag Then
            Dim rowIndex As Integer = e.RowIndex
            Dim colIndex As Integer = e.ColIndex
            e.Handled = True
            Dim enabledCellFound As Boolean = True
            While enabledCellFound
                enabledCellFound = CurrentCell.QueryNextEnabledCell(e.Direction, rowIndex, colIndex)
                If enabledCellFound Then
                    If Not Me(rowIndex, colIndex).ReadOnly Then
                        Exit While
                    End If
                    Select Case e.Direction
                        Case GridDirectionType.Left
                            colIndex -= 1
                        Case GridDirectionType.Right
                            colIndex += 1
                        Case GridDirectionType.Up
                            rowIndex -= 1
                        Case GridDirectionType.Down
                            rowIndex += 1
                        Case Else
                            enabledCellFound = False
                    End Select
                End If
            End While

            If enabledCellFound Then
                e.RowIndex = rowIndex
                e.ColIndex = colIndex
            End If
            e.Result = enabledCellFound
        End If
        MyBase.OnQueryNextCurrentCellPosition(e)
    End Sub 'OnQueryNextCurrentCellPosition

    Public Shared Sub SetupGridModel(ByVal model As GridModel)

        GridFactoryProvider.Init(New GridCellModelFactory())
        model.Options.AlphaBlendSelectionColor = Color.FromArgb(64, SystemColors.Highlight)
        model.Options.AllowSelection = GridSelectionFlags.Any

        'model.BaseStylesMap.RegisterStandardStyles();
        model.CommandStack.Enabled = True
        AddHandler model.QueryCellModel, AddressOf ModelQueryCellModel
        model.OperationFeedbackListener = New DelayedStatusDialog(model)

        model.BeginInit()
        model.RowCount = 1000
        model.ColCount = 100
        model.Rows.DefaultSize = 19
        model.Cols.DefaultSize = 65
        model.RowHeights(0) = 21
        model.ColWidths(0) = 35
        model.ColWidths(3) = 85
        model.Options.ControllerOptions = GridControllerOptions.All
        model.Options.DataObjectConsumerOptions = GridDataObjectConsumerOptions.All


        Dim standard As GridStyleInfo = model.BaseStylesMap("Standard").StyleInfo
        Dim header As GridStyleInfo = model.BaseStylesMap("Header").StyleInfo
        Dim rowHeader As GridStyleInfo = model.BaseStylesMap("Row Header").StyleInfo
        Dim colHeader As GridStyleInfo = model.BaseStylesMap("Column Header").StyleInfo
        model.Properties.GridLineColor = Color.FromArgb(65, 57, 73, 122)

        Dim boldFont As New GridFontInfo()
        boldFont.Bold = True
        boldFont.Size = 11
        boldFont.Facename = "Helvetica"

        standard.TextColor = Color.FromArgb(0, 21, 84)

        Dim boldFontStyle As New GridStyleInfo()
        boldFontStyle.TextColor = Color.FromArgb(238, 122, 3)
        boldFontStyle.Font = boldFont

        header.Interior = New BrushInfo(GradientStyle.Vertical, Color.FromArgb(65, 238, 234, 216), Color.FromArgb(65, 203, 199, 184))
        rowHeader.Interior = New BrushInfo(GradientStyle.Horizontal, Color.FromArgb(65, 238, 234, 216), Color.FromArgb(65, 203, 199, 184))

        Dim showImageList As Boolean = False
        If showImageList Then
            Dim imageList As New imageList()
            imageList.Images.Add(SystemIcons.Warning.ToBitmap())
            imageList.Images.Add(SystemIcons.Application.ToBitmap())
            imageList.Images.Add(SystemIcons.Asterisk.ToBitmap())
            imageList.Images.Add(SystemIcons.Error.ToBitmap())
            imageList.Images.Add(SystemIcons.Exclamation.ToBitmap())
            imageList.Images.Add(SystemIcons.Hand.ToBitmap())
            imageList.Images.Add(SystemIcons.Information.ToBitmap())
            imageList.Images.Add(SystemIcons.Question.ToBitmap())
            standard.ImageList = imageList
        End If

        Dim USStates As New ArrayList()
        USStates.Add(New USState("Alabama", "AL", 0))
        USStates.Add(New USState("Washington", "WA", 1))
        USStates.Add(New USState("West Virginia", "WV", 2))
        USStates.Add(New USState("Wisconsin", "WI", 3))
        USStates.Add(New USState("Wyoming", "WY", 4))
        USStates.Add(New USState("XAlabama", "AAL", 5))
        USStates.Add(New USState("XWashington", "AWA", 6))
        USStates.Add(New USState("XWest Virginia", "AWV", 1))
        USStates.Add(New USState("XWisconsin", "AWI", 2))
        USStates.Add(New USState("XWyoming", "AWY", 3))
        USStates.Add(New USState("QAlabama", "BAL", 4))
        USStates.Add(New USState("QWashington", "BWA", 5))
        USStates.Add(New USState("QWest Virginia", "BWV", 6))
        USStates.Add(New USState("QWisconsin", "BWI", 0))
        USStates.Add(New USState("QWyoming", "BWY", 1))

        standard.DataSource = USStates
        standard.DisplayMember = "LongName"
        standard.ValueMember = "ShortName"

        Dim type As type = GetType(Bitmap)
        Dim stream As System.IO.Stream = GetType(SampleGrid).Module.Assembly.GetManifestResourceStream((ApplicationMain.manifestNamespace + "logo_105_65.bmp"))
        Dim bm As New Bitmap(stream)
        Dim logoList As New imageList()
        logoList.Images.Add(bm)
        logoList.ImageSize = bm.Size
        logoList.ColorDepth = ColorDepth.Depth32Bit

        Dim rowIndex As Integer = 2
        Dim colIndex As Integer = 1

        Dim boldfontRowHeight As Integer = 0


        model.Options.FloatCellsMode = GridFloatCellsMode.OnDemandCalculation


        model.CoveredRanges.Add(GridRangeInfo.Cells(2, 2, 4, 4))

        model(2, 2).Text = "Grid Demo"
        model(2, 2).Font.Bold = True
        model(2, 2).Font.Size = 16
        model(2, 2).HorizontalAlignment = GridHorizontalAlignment.Center
        model(2, 2).VerticalAlignment = GridVerticalAlignment.Middle
        model(2, 2).CellType = "Static"
        model(2, 2).Borders.All = New GridBorder(GridBorderStyle.Solid, Color.FromArgb(100, 238, 122, 3))
        model(2, 2).Interior = New BrushInfo(GradientStyle.PathEllipse, Color.FromArgb(100, 57, 73, 122), Color.FromArgb(65, 237, 240, 247))
        rowIndex += 5

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Interior"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights.ResizeToFit(GridRangeInfo.Cell(rowIndex, 1), GridResizeToFitOptions.ResizeCoveredCells Or GridResizeToFitOptions.NoShrinkSize)
        boldfontRowHeight = model.RowHeights(rowIndex)
        rowIndex += 1

        colIndex = 1
        Dim index As Integer
        For Each index In New Integer() {1, 2, 3, 4, 5, 6}
            model(rowIndex, colIndex).ImageIndex = index
            colIndex += 1
        Next index
        rowIndex += 1


        colIndex = 1
        Dim color1 As Color
        For Each color1 In New Color() {Color.FromArgb(65, 0, 0, 51), Color.FromArgb(65, 252, 172, 38), Color.FromArgb(65, 33, 67, 126), Color.FromArgb(65, 102, 110, 152), Color.FromArgb(65, 204, 212, 230), Color.FromArgb(65, 237, 240, 247)}
            model(rowIndex, colIndex).Interior = New BrushInfo(color1)
            colIndex += 1
        Next color1

        rowIndex += 1
        colIndex = 1
        Dim pattern As PatternStyle
        For Each pattern In New PatternStyle() {PatternStyle.Percent05, PatternStyle.Percent20, PatternStyle.Percent50, PatternStyle.Percent75, PatternStyle.DarkDownwardDiagonal, PatternStyle.LightDownwardDiagonal}
            model(rowIndex, colIndex).Interior = New BrushInfo(pattern, Color.FromArgb(65, 237, 240, 247), Color.FromArgb(65, 57, 73, 122))
            colIndex += 1
        Next pattern

        rowIndex += 1
        colIndex = 1
        Dim gradient As GradientStyle
        For Each gradient In New GradientStyle() {GradientStyle.BackwardDiagonal, GradientStyle.ForwardDiagonal, GradientStyle.PathEllipse, GradientStyle.PathRectangle, GradientStyle.Vertical, GradientStyle.Horizontal}
            model(rowIndex, colIndex).Interior = New BrushInfo(gradient, Color.FromArgb(65, 57, 73, 122), Color.FromArgb(65, 237, 240, 247))
            model(rowIndex + 1, colIndex).Interior = New BrushInfo(gradient, Color.FromArgb(65, 237, 240, 247), Color.FromArgb(65, 51, 51, 102))
            colIndex += 1
        Next gradient

        rowIndex += 1
        rowIndex += 1
        rowIndex += 1
        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Font"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        Dim size As Integer
        For Each size In New Integer() {6, 7, 8, 10, 12, 14}
            model(rowIndex, colIndex).Font.Size = size
            model(rowIndex, colIndex).Text = size.ToString()
            colIndex += 1
        Next size
        model.RowHeights.ResizeToFit(GridRangeInfo.Row(rowIndex), GridResizeToFitOptions.ResizeCoveredCells Or GridResizeToFitOptions.NoShrinkSize)

        rowIndex += 1
        colIndex = 1
        Dim fontStyle As fontStyle
        For Each fontStyle In New fontStyle() {fontStyle.Bold, fontStyle.Italic, fontStyle.Regular, fontStyle.Strikeout, fontStyle.Underline, fontStyle.Bold Or fontStyle.Italic Or fontStyle.Regular}
            model(rowIndex, colIndex).Font.FontStyle = fontStyle
            model(rowIndex, colIndex).Text = [Enum].Format(GetType(fontStyle), fontStyle, "G")
            colIndex += 1
        Next fontStyle

        rowIndex += 1
        rowIndex += 1
        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "TextColor"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        colIndex = 1
        Dim color2 As Color
        For Each color2 In New Color() {Color.Black, Color.Red, Color.Blue, Color.Green, Color.Yellow, Color.DimGray}
            model(rowIndex, colIndex).TextColor = color2
            model(rowIndex, colIndex).Text = ColorConvert.ColorToString(color2, True)
            colIndex += 1
        Next color2

        rowIndex += 1
        rowIndex += 1

        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Number Formats"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        colIndex = 1
        Dim format2 As String
        For Each format2 In New String() {"0.00", "C", "0.00;(0.00)", "###0.##%", "#0.#E+00", "10:##,##0.#"}

            model(rowIndex, colIndex).Format = format2
            model(rowIndex, colIndex).CellValue = Math.PI
            model(rowIndex, colIndex).CellValueType = GetType(Double)
            colIndex += 1
        Next format2

        rowIndex += 1
        rowIndex += 1

        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "DateTime Formats"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        colIndex = 1
        Dim format As String
        For Each format In New String() {"d", "D", "f", "dddd, dd MMMM yyyy", "t", "s"}

            model(rowIndex, colIndex).Format = format
            model(rowIndex, colIndex).CellValue = DateTime.Now
            model(rowIndex, colIndex).CellValueType = GetType(DateTime)
            colIndex += 1
        Next format

        rowIndex += 1
        rowIndex += 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Validation"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        model(rowIndex, 1).Text = "Positive numbers"
        model.RowStyles(rowIndex).ValidateValue.NumberRequired = True
        model.RowStyles(rowIndex).ValidateValue.Minimum = 0
        model.RowStyles(rowIndex).ValidateValue.Maximum = Single.NaN
        model.RowStyles(rowIndex).ValidateValue.ErrorMessage = "Please enter a number greater than 0!"
        rowIndex += 1

        model(rowIndex, 1).Text = "Validation (1-100 valid range)"
        model.RowStyles(rowIndex).ValidateValue = New GridCellValidateValueInfo(True, 1, 100, "Please enter a number between 1 and 100!")
        rowIndex += 1
        rowIndex += 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Cell Types"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        colIndex = 1

        model.TableStyle.CheckBoxOptions = New GridCheckBoxCellInfo("True", "False", "", False)
        model(rowIndex, 1).Text = "CheckBox"
        model(rowIndex, 3).CellType = "CheckBox"
        model(rowIndex, 3).TriState = False
        model(rowIndex, 3).Text = "True"
        model(rowIndex, 3).VerticalAlignment = GridVerticalAlignment.Middle
        model(rowIndex, 3).Description = "Enabled"
        model(rowIndex, 4).CellType = "CheckBox"
        model(rowIndex, 4).Text = "False"
        model(rowIndex, 4).Enabled = False
        model(rowIndex, 4).Description = "Disabled"
        model(rowIndex, 4).VerticalAlignment = GridVerticalAlignment.Middle
        model(rowIndex, 5).CellType = "CheckBox"
        model(rowIndex, 5).TriState = True
        model(rowIndex, 5).Description = "TriState"
        model(rowIndex, 5).VerticalAlignment = GridVerticalAlignment.Middle
        rowIndex += 1

        rowIndex += 1
        rowIndex += 1

        model(rowIndex, 1).Text = "DragButton"
        model(rowIndex, 3).Text = "Drag"
        model(rowIndex, 3).CellType = "DragButton"
        rowIndex += 1

        model.TableStyle.NumericUpDown = New GridNumericUpDownCellInfo(0, 25, 1, 1, True)
        model(rowIndex, 1).Text = "NumericUpDown"
        ' Wrapping, Range 0-1000
        model(rowIndex, 3).CellType = "NumericUpDown"
        model(rowIndex, 3).NumericUpDown = New GridNumericUpDownCellInfo(0, 1000, 0, 1, True)
        ' Disabled
        model(rowIndex, 4).CellType = "NumericUpDown"
        model(rowIndex, 4).Enabled = False
        model(rowIndex, 4).Text = "5"
        ' No wrapping, Range 1-20
        model(rowIndex, 5).CellType = "NumericUpDown"
        model(rowIndex, 5).NumericUpDown = New GridNumericUpDownCellInfo(1, 20, 1, 1, False)
        rowIndex += 1

        model(rowIndex, 1).Text = "PushButton"
        model(rowIndex, 3).CellType = "PushButton"
        model(rowIndex, 3).Description = "Enabled"
        model(rowIndex, 4).CellType = "PushButton"
        model(rowIndex, 4).Enabled = False
        model(rowIndex, 4).Clickable = False
        model(rowIndex, 4).Description = "Disabled"
        rowIndex += 1

        Dim items As New StringCollection()
        items.Add("One")
        items.Add("Two")
        items.Add("Three")
        items.Add("Four")
        items.Add("Five")
        items.Add("Six")
        items.Add("Seven")
        items.Add("Eight")
        items.Add("Nine")
        items.Add("Ten")

        model(rowIndex, 1).Text = "ComboBox (Edit)"

        model(rowIndex, 3).CellType = "ComboBox"
        model(rowIndex, 3).ChoiceList = items
        ' Disabled
        model(rowIndex, 4).CellType = "ComboBox"
        model(rowIndex, 4).ChoiceList = items
        model(rowIndex, 4).Enabled = False
        model(rowIndex, 4).CellValue = items(5)
        rowIndex += 1

        model(rowIndex, 1).Text = "ComboBox (List)"

        model(rowIndex, 3).CellType = "ComboBox"
        model(rowIndex, 3).ChoiceList = items
        model(rowIndex, 3).ExclusiveChoiceList = True ' tell combobox to behave like CBS_DROPDOWNSTYLE
        ' Disabled
        model(rowIndex, 4).CellType = "ComboBox"
        model(rowIndex, 4).ChoiceList = items
        model(rowIndex, 4).Enabled = False
        model(rowIndex, 4).ExclusiveChoiceList = True ' tell combobox to behave like CBS_DROPDOWNSTYLE
        model(rowIndex, 4).CellValue = items(5)
        rowIndex += 1
        rowIndex += 1

        model(rowIndex, 1).Text = "GridListControl (Edit)"
        model(rowIndex, 3).CellType = "GridListControl"
        model(rowIndex, 3).DataSource = USStates

        ' Disabled
        model(rowIndex, 4).CellType = "GridListControl"
        model(rowIndex, 4).DataSource = USStates
        rowIndex += 1

        model(rowIndex, 1).Text = "GridListControl (List)"

        model(rowIndex, 3).CellType = "GridListControl"
        model(rowIndex, 3).DataSource = USStates
        model(rowIndex, 3).ExclusiveChoiceList = True ' tell combobox to behave like CBS_DROPDOWNSTYLE
        rowIndex += 1
        rowIndex += 1


        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "DropDown Cells"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        '
        model(rowIndex, 1).Text = "Generic DropDown"
        model(rowIndex, 3).CellType = "DropDown"
        rowIndex += 1

        model(rowIndex, 1).Text = "MonthCalendar"
        model(rowIndex, 3).CellType = "MonthCalendar"
        rowIndex += 1

        model(rowIndex, 1).Text = "ColorEdit"
        model(rowIndex, 3).CellType = "ColorEdit"
        rowIndex += 1

        model(rowIndex, 1).Text = "DropDownGrid"
        model(rowIndex, 3).CellType = "DropDownGrid"
        rowIndex += 1
        rowIndex += 1

        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Borders"
        model.ChangeCells(GridRangeInfo.Cell(rowIndex, 1), boldFontStyle)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        Dim border2 As GridBorder
        For Each border2 In New GridBorder() {New GridBorder(GridBorderStyle.Solid, Color.FromArgb(57, 73, 122)), New GridBorder(GridBorderStyle.Dotted, Color.FromArgb(238, 122, 3)), New GridBorder(GridBorderStyle.DashDot, Color.FromArgb(57, 73, 122)), New GridBorder(GridBorderStyle.DashDotDot, Color.FromArgb(238, 122, 3)), New GridBorder(GridBorderStyle.Dashed, Color.FromArgb(57, 73, 122), GridBorderWeight.Medium), New GridBorder(GridBorderStyle.Dotted, Color.FromArgb(238, 122, 3), GridBorderWeight.Medium)}
            model(rowIndex, colIndex).Borders.Bottom = border2
            model(rowIndex, colIndex).Borders.Right = border2
            model(rowIndex, colIndex).Text = border2.ToString()
            colIndex += 1
        Next border2

        rowIndex += 1
        colIndex = 1
        Dim border As GridBorder
        For Each border In New GridBorder() {New GridBorder(GridBorderStyle.Dashed, Color.FromArgb(57, 73, 122), GridBorderWeight.Thick), New GridBorder(GridBorderStyle.Dotted, Color.FromArgb(238, 122, 3), GridBorderWeight.Thick), New GridBorder(GridBorderStyle.DashDot, Color.FromArgb(57, 73, 122), GridBorderWeight.Medium), New GridBorder(GridBorderStyle.DashDotDot, Color.FromArgb(238, 122, 3), GridBorderWeight.Medium), New GridBorder(GridBorderStyle.Solid, Color.FromArgb(57, 73, 122), GridBorderWeight.Thick), New GridBorder(GridBorderStyle.Solid, Color.FromArgb(238, 122, 3), GridBorderWeight.Medium)}
            model(rowIndex, colIndex).Borders.Bottom = border
            model(rowIndex, colIndex).Borders.Right = border
            model(rowIndex, colIndex).Text = border.ToString()
            colIndex += 1
        Next border

        rowIndex += 1
        rowIndex += 1
        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Orientation"
        model(rowIndex, 1).ModifyStyle(boldFontStyle, StyleModifyType.Override)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        Dim orientation As Integer
        For Each orientation In New Integer() {0, 45, 60, 90, 180, -90}
            Dim style1 As GridStyleInfo = model(rowIndex, colIndex)
            ' style1 is a direct reference to model[rowIndex, colIndex]; 
            ' changes will be propagated back to GridData
            style1.HorizontalAlignment = GridHorizontalAlignment.Center
            style1.VerticalAlignment = GridVerticalAlignment.Middle
            Dim font As GridFontInfo = style1.Font
            font.Orientation = orientation
            font.Bold = True
            style1.Text = "Angle = " + font.Orientation.ToString()
            colIndex += 1
        Next orientation

        rowIndex += 1
        rowIndex += 1
        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Enabled"
        model(rowIndex, 1).ModifyStyle(boldFontStyle, StyleModifyType.Override)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        Dim enabled1 As Boolean
        For Each enabled1 In New Boolean() {True, False, True, False, False, True}
            model(rowIndex, colIndex).Text = IIf(enabled1, "YES", "NO")
            model(rowIndex, colIndex).Interior = New BrushInfo(CType(IIf(enabled1, Color.FromArgb(65, 57, 73, 122), Color.FromArgb(65, 238, 122, 3)), Color))
            model(rowIndex, colIndex).Enabled = enabled1
            colIndex += 1
        Next enabled1

        colIndex = 1
        rowIndex += 1

        Dim enabled2 As Boolean
        For Each enabled2 In New Boolean() {True, True, False, False, False, True}
            model(rowIndex, colIndex).Text = IIf(enabled2, "YES", "NO")
            model(rowIndex, colIndex).Interior = New BrushInfo(CType(IIf(enabled2, Color.FromArgb(65, 57, 73, 122), Color.FromArgb(65, 238, 122, 3)), Color))
            model(rowIndex, colIndex).Enabled = enabled2
            colIndex += 1
        Next enabled2

        rowIndex += 1
        rowIndex += 1
        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "AutoSize"
        model(rowIndex, 1).ModifyStyle(boldFontStyle, StyleModifyType.Override)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        model.RowStyles(rowIndex).AutoSize = True
        model.RowStyles(rowIndex).Text = "Row height will increase while you type text"
        model.RowStyles(rowIndex).Interior = New BrushInfo(Color.Beige)

        rowIndex += 1
        rowIndex += 1
        colIndex = 1

        model.CoveredRanges.Add(GridRangeInfo.Cells(rowIndex, 1, rowIndex, 2))
        model(rowIndex, 1).Text = "Read Only"
        model(rowIndex, 1).ModifyStyle(boldFontStyle, StyleModifyType.Override)
        model.RowHeights(rowIndex) = boldfontRowHeight
        rowIndex += 1

        model.RowStyles(rowIndex).ReadOnly = True
        model.RowStyles(rowIndex).Interior = New BrushInfo(Color.LightGray)
        model.RowStyles(rowIndex).Text = "ReadOnly"
        rowIndex += 1
        rowIndex += 1


        model.Options.ExcelLikeCurrentCell = False
        model.Options.ExcelLikeSelectionFrame = False
        model.Options.ActivateCurrentCellBehavior = GridCellActivateAction.ClickOnCell
        model.Properties.MarkColHeader = True
        model.Properties.MarkRowHeader = True

        model.RowHeights.ResizeToFit(GridRangeInfo.Cells(2, 1, rowIndex, 3), GridResizeToFitOptions.ResizeCoveredCells Or GridResizeToFitOptions.NoShrinkSize)
        model.EndInit()
    End Sub 'SetupGridModel


    Shared Sub ModelQueryCellModel(ByVal sender As Object, ByVal e As GridQueryCellModelEventArgs)
        If e.CellModel Is Nothing Then
            Dim pGridCellObjectFactory As IGridCellModelFactory = GridFactoryProvider.CellModelFactory

            If Not (pGridCellObjectFactory Is Nothing) Then
                e.CellModel = pGridCellObjectFactory.CreateCellModel(e.CellType, CType(sender, GridModel))
            End If
        End If
    End Sub 'ModelQueryCellModel

    Protected Overloads Overrides Sub OnKeyDown(ByVal e As KeyEventArgs)
        Dim runTests As Boolean = False
        If runTests Then
            If e.KeyData = Keys.T Then
                ' Let's run some style tests.
                Dim style As New GridStyleInfo()
                MessageBox.Show(style.ToString("d"), "Empty Style")

                MessageBox.Show(GridStyleInfo.Default.ToString(), "GridStyleInfo.Default.ToString()")

                MessageBox.Show(Model.TableStyle.ToString(), "TableStyle.ToString()")
                MessageBox.Show(Model.TableStyle.ToString("d"), "TableStyle.ToString(""d"")")

                MessageBox.Show(Model(1, 1).ToString(), "Model[1, 1].ToString()")
                MessageBox.Show(Model(1, 1).ToString("d"), "Model[1, 1].ToString(""d"")")

                MessageBox.Show(Model.RowStyles(1).ToString(), "RowStyles[1].ToString()")
                MessageBox.Show(Model.RowStyles(1).ToString("d"), "RowStyles[1].ToString(""d"")")

                Dim t As String = Model.TableStyle.ToString()
                Dim parseStyle As New GridStyleInfo()
                parseStyle.ParseString(t)
                MessageBox.Show(parseStyle.ToString(), "parseStyle.ToString()")
            End If

            If e.KeyData = Keys.F1 Then
                ' TODO: hangs when you change base style in property grid.
                Dim form As New StyleInfoForm(Model.ColStyles(1))
                form.ShowDialog()
                Refresh()
            End If

            If e.KeyData = Keys.F3 Then
                CurrentCell.MoveTo(5, 4, GridSetCurrentCellOptions.SetFocus)
                Return
            End If

            If e.KeyData = Keys.F4 Then
                ' TODO: hangs when you change base style in property grid.
                Dim form As New StyleInfoForm(Model(0, 0))
                form.ShowDialog()
                Refresh()
            End If

            If e.KeyData = Keys.F5 Then
                Model.Selections.Clear()
            End If

            If e.KeyData = Keys.F6 Then
                Model.Selections.Add(GridRangeInfo.Cells(2, 2, 10, 4))
            End If

            If e.KeyData = Keys.M Then
                ' TODO: Show Default value for base style in property grid
                Dim ce As New GridBaseStyleCollectionEditor(GetType(GridBaseStylesMap))
                Dim esc As New WindowsFormsEditorServiceContainer(Nothing)
                Dim pd As PropertyDescriptor = TypeDescriptor.GetProperties(Model)("GridBaseStylesMap")
                Dim tdc As New TypeDescriptorContext(Model, pd)
                tdc.ServiceProvider = esc
                Dim v As Object = ce.EditValue(tdc, esc, Model.BaseStylesMap.Clone())
                If esc.DialogResult = DialogResult.OK Then
                    Model.BaseStylesMap = CType(v, GridBaseStylesMap)
                End If
            End If
        End If

        MyBase.OnKeyDown(e)
    End Sub 'OnKeyDown

    
 ' Sample code for inserting a work sheet.
    Protected Overrides Sub OnCellDoubleClick(ByVal e As GridCellClickEventArgs)
        Dim form As WorkbookForm = CType(FindForm(), WorkbookForm)

        If Not (form Is Nothing) Then
            Dim workbook As WorkbookModel = form.WorkbookView.Workbook

            Dim sheet3 As SampleGridModel = New SampleGridModel()
            SampleGrid.SetupGridModel(sheet3)

            Dim workSheetModel As New workSheetModel(workbook, "Sheet 3", sheet3)
            workbook.Worksheets.Add(workSheetModel)

            form.WorkbookView.ActivePage = form.WorkbookView.TabBarPages(workbook.Worksheets.Count - 1)
        End If

        MyBase.OnCellDoubleClick(e)
    End Sub 'OnCellDoubleClick
End Class 'SampleGrid

<Serializable()> _
Public Class USState
    Private myShortName As String
    Private myLongName As String
    Private _imageIndex As Integer


    Public Sub New(ByVal strLongName As String, ByVal strShortName As String, ByVal _imageIndex As Integer)
        Me.myShortName = strShortName
        Me.myLongName = strLongName
        Me._imageIndex = _imageIndex
    End Sub 'New


    Public ReadOnly Property ShortName() As String
        Get
            Return myShortName
        End Get
    End Property


    Public ReadOnly Property LongName() As String

        Get
            Return myLongName
        End Get
    End Property


    Public Property ImageIndex() As Integer
        Get
            Return _imageIndex
        End Get
        Set(ByVal Value As Integer)
            _imageIndex = Value
        End Set
    End Property



    Public Overrides Function ToString() As String
        Return Me.ShortName + " - " + Me.LongName
    End Function 'ToString


End Class 'USState
